@@ -834,7 +834,7 @@ IOSObjectArray *IOSClass_NewInterfacesFromProtocolList(
 }
 
 - (IOSObjectArray *)getAnnotations {
-  NSMutableArray *array = [[NSMutableArray alloc] init];
+  NSMutableArray *array = AUTORELEASE([[NSMutableArray alloc] init]);
   IOSObjectArray *declared = [self getDeclaredAnnotations];
   for (jint i = 0; i < declared->size_; i++) {
     [array addObject:declared->buffer_[i]];
@@ -859,7 +859,6 @@ IOSObjectArray *IOSClass_NewInterfacesFromProtocolList(
   }
   IOSObjectArray *result =
       [IOSObjectArray arrayWithNSArray:array type:JavaLangAnnotationAnnotation_class_()];
-  [array release];
   return result;
 }
 
@@ -923,6 +922,10 @@ IOSObjectArray *IOSClass_NewInterfacesFromProtocolList(
   return nil;
 }
 
+- (NSString *)getPackageName {
+  return JreClassPackageName(metadata_);
+}
+
 - (id)getClassLoader {
   return JavaLangClassLoader_getSystemClassLoader();
 }
@@ -1281,7 +1284,7 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
 // Generated by running the translator over the java.lang.Class stub file.
 + (const J2ObjcClassInfo *)__metadata {
   static J2ObjcMethodInfo methods[] = {
-    { NULL, NULL, 0x1, -1, -1, -1, -1, -1, -1 },
+    { NULL, NULL, 0x2, -1, -1, -1, -1, -1, -1 },
     { NULL, "LIOSClass;", 0x9, 0, 1, 2, 3, -1, -1 },
     { NULL, "LIOSClass;", 0x9, 0, 4, 2, 5, -1, -1 },
     { NULL, "LIOSClass;", 0x1, 6, 7, -1, 8, -1, -1 },
@@ -1296,11 +1299,11 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     { NULL, "LJavaLangReflectConstructor;", 0x81, 16, 17, 18, 19, -1, -1 },
     { NULL, "[LJavaLangReflectConstructor;", 0x1, -1, -1, 20, 21, -1, -1 },
     { NULL, "[LJavaLangAnnotationAnnotation;", 0x1, -1, -1, -1, -1, -1, -1 },
-    { NULL, "[LIOSClass;", 0x1, -1, -1, 20, 14, -1, -1 },
+    { NULL, "[LIOSClass;", 0x1, -1, -1, -1, 14, -1, -1 },
     { NULL, "LJavaLangReflectConstructor;", 0x81, 22, 17, 18, 19, -1, -1 },
     { NULL, "[LJavaLangReflectConstructor;", 0x1, -1, -1, 20, 21, -1, -1 },
     { NULL, "LJavaLangReflectField;", 0x1, 23, 1, 24, -1, -1, -1 },
-    { NULL, "[LJavaLangReflectField;", 0x1, -1, -1, 20, -1, -1, -1 },
+    { NULL, "[LJavaLangReflectField;", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LJavaLangReflectMethod;", 0x81, 25, 26, 18, 27, -1, -1 },
     { NULL, "[LJavaLangReflectMethod;", 0x1, -1, -1, 20, -1, -1, -1 },
     { NULL, "LIOSClass;", 0x1, -1, -1, -1, 15, -1, -1 },
@@ -1319,6 +1322,7 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     { NULL, "I", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LNSString;", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LJavaLangPackage;", 0x1, -1, -1, -1, -1, -1, -1 },
+    { NULL, "LNSString;", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LJavaSecurityProtectionDomain;", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LJavaNetURL;", 0x1, 32, 1, -1, -1, -1, -1 },
     { NULL, "LJavaIoInputStream;", 0x1, 33, 1, -1, -1, -1, -1 },
@@ -1342,12 +1346,13 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     { NULL, "LNSString;", 0x1, 43, -1, -1, -1, -1, -1 },
     { NULL, "[LJavaLangAnnotationAnnotation;", 0x1, 44, 7, -1, 45, -1, -1 },
     { NULL, "[LJavaLangAnnotationAnnotation;", 0x1, 46, 7, -1, 45, -1, -1 },
-    { NULL, "LJavaLangAnnotationAnnotation;", 0x1, 47, 7, -1, 48, -1, -1 },
+    { NULL, "LJavaLangAnnotationAnnotation;", 0x1, 47, 7, -1, 13, -1, -1 },
     { NULL, "LNSString;", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LNSString;", 0x1, -1, -1, -1, -1, -1, -1 },
   };
   #pragma clang diagnostic push
   #pragma clang diagnostic ignored "-Wobjc-multiple-method-names"
+  #pragma clang diagnostic ignored "-Wundeclared-selector"
   methods[0].selector = @selector(init);
   methods[1].selector = @selector(forName:);
   methods[2].selector = @selector(forName:initialize:classLoader:);
@@ -1386,32 +1391,33 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
   methods[35].selector = @selector(getModifiers);
   methods[36].selector = @selector(getName);
   methods[37].selector = @selector(getPackage);
-  methods[38].selector = @selector(getProtectionDomain);
-  methods[39].selector = @selector(getResource:);
-  methods[40].selector = @selector(getResourceAsStream:);
-  methods[41].selector = @selector(getSigners);
-  methods[42].selector = @selector(getSimpleName);
-  methods[43].selector = @selector(getSuperclass);
-  methods[44].selector = @selector(getTypeParameters);
-  methods[45].selector = @selector(isAnnotation);
-  methods[46].selector = @selector(isAnnotationPresentWithIOSClass:);
-  methods[47].selector = @selector(isAnonymousClass);
-  methods[48].selector = @selector(isArray);
-  methods[49].selector = @selector(isAssignableFrom:);
-  methods[50].selector = @selector(isEnum);
-  methods[51].selector = @selector(isInstance:);
-  methods[52].selector = @selector(isInterface);
-  methods[53].selector = @selector(isLocalClass);
-  methods[54].selector = @selector(isMemberClass);
-  methods[55].selector = @selector(isPrimitive);
-  methods[56].selector = @selector(isSynthetic);
-  methods[57].selector = @selector(newInstance);
-  methods[58].selector = @selector(description);
-  methods[59].selector = @selector(getDeclaredAnnotationsByTypeWithIOSClass:);
-  methods[60].selector = @selector(getAnnotationsByTypeWithIOSClass:);
-  methods[61].selector = @selector(getDeclaredAnnotationWithIOSClass:);
-  methods[62].selector = @selector(getTypeName);
-  methods[63].selector = @selector(toGenericString);
+  methods[38].selector = @selector(getPackageName);
+  methods[39].selector = @selector(getProtectionDomain);
+  methods[40].selector = @selector(getResource:);
+  methods[41].selector = @selector(getResourceAsStream:);
+  methods[42].selector = @selector(getSigners);
+  methods[43].selector = @selector(getSimpleName);
+  methods[44].selector = @selector(getSuperclass);
+  methods[45].selector = @selector(getTypeParameters);
+  methods[46].selector = @selector(isAnnotation);
+  methods[47].selector = @selector(isAnnotationPresentWithIOSClass:);
+  methods[48].selector = @selector(isAnonymousClass);
+  methods[49].selector = @selector(isArray);
+  methods[50].selector = @selector(isAssignableFrom:);
+  methods[51].selector = @selector(isEnum);
+  methods[52].selector = @selector(isInstance:);
+  methods[53].selector = @selector(isInterface);
+  methods[54].selector = @selector(isLocalClass);
+  methods[55].selector = @selector(isMemberClass);
+  methods[56].selector = @selector(isPrimitive);
+  methods[57].selector = @selector(isSynthetic);
+  methods[58].selector = @selector(newInstance);
+  methods[59].selector = @selector(description);
+  methods[60].selector = @selector(getDeclaredAnnotationsByTypeWithIOSClass:);
+  methods[61].selector = @selector(getAnnotationsByTypeWithIOSClass:);
+  methods[62].selector = @selector(getDeclaredAnnotationWithIOSClass:);
+  methods[63].selector = @selector(getTypeName);
+  methods[64].selector = @selector(toGenericString);
   #pragma clang diagnostic pop
   static const J2ObjcFieldInfo fields[] = {
     { "serialVersionUID", "J", .constantValue.asLong = IOSClass_serialVersionUID, 0x1a, -1, -1, -1,
@@ -1428,8 +1434,8 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     "LJavaLangNoSuchMethodException;LJavaLangSecurityException;",
     "([Ljava/lang/Class<*>;)Ljava/lang/reflect/Constructor<TT;>;", "LJavaLangSecurityException;",
     "()[Ljava/lang/reflect/Constructor<*>;", "getDeclaredConstructor", "getDeclaredField",
-    "LJavaLangNoSuchFieldException;LJavaLangSecurityException;", "getDeclaredMethod",
-    "LNSString;[LIOSClass;", "(Ljava/lang/String;[Ljava/lang/Class<*>;)Ljava/lang/reflect/Method;",
+    "LJavaLangNoSuchFieldException;", "getDeclaredMethod", "LNSString;[LIOSClass;",
+    "(Ljava/lang/String;[Ljava/lang/Class<*>;)Ljava/lang/reflect/Method;",
     "()Ljava/lang/reflect/Constructor<*>;", "()[TT;", "getField", "getMethod", "getResource",
     "getResourceAsStream", "()Ljava/lang/Class<-TT;>;",
     "()[Ljava/lang/reflect/TypeVariable<Ljava/lang/Class<TT;>;>;", "isAnnotationPresent",
@@ -1437,12 +1443,13 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     "(Ljava/lang/Class<*>;)Z", "isInstance",
     "LJavaLangInstantiationException;LJavaLangIllegalAccessException;", "()TT;", "toString",
     "getDeclaredAnnotationsByType",
-    "<T::Ljava/lang/annotation/Annotation;>(Ljava/lang/Class<TT;>;)[TT;", "getAnnotationsByType",
-    "getDeclaredAnnotation", "<T::Ljava/lang/annotation/Annotation;>(Ljava/lang/Class<TT;>;)TT;",
-    "<T:Ljava/lang/Object;>Ljava/lang/Object;Ljava/lang/reflect/AnnotatedElement;"
-    "Ljava/lang/reflect/GenericDeclaration;Ljava/io/Serializable;Ljava/lang/reflect/Type;" };
+    "<A::Ljava/lang/annotation/Annotation;>(Ljava/lang/Class<TA;>;)[TA;", "getAnnotationsByType",
+    "getDeclaredAnnotation",
+    "<T:Ljava/lang/Object;>Ljava/lang/Object;Ljava/io/Serializable;"
+    "Ljava/lang/reflect/GenericDeclaration;Ljava/lang/reflect/Type;"
+    "Ljava/lang/reflect/AnnotatedElement;" };
   static const J2ObjcClassInfo _IOSClass = {
-    "Class", "java.lang", ptrTable, methods, fields, 7, 0x11, 64, 1, -1, -1, -1, 49, -1 };
+    "Class", "java.lang", ptrTable, methods, fields, 7, 0x11, 65, 1, -1, -1, -1, 48, -1 };
   return &_IOSClass;
 }
 
