@@ -25,28 +25,14 @@
 
 package java.net;
 
-import android.system.ErrnoException;
+import static libcore.io.OsConstants.*;
 
-import java.io.FileDescriptor;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 import java.util.NoSuchElementException;
 
-import libcore.io.IoUtils;
-import libcore.io.Libcore;
-/* J2ObjC removed.
-import android.system.StructIfaddrs;
-import sun.security.action.*;
-import java.security.AccessController;
- */
-
-import static libcore.io.OsConstants.*;
-
 // Android-note: NetworkInterface has been rewritten to avoid native code.
 // Fix upstream bug not returning link-down interfaces. http://b/26238832
 // Android-added: Document restrictions for targetSdkVersion >= R. http://b/141455849
@@ -60,7 +46,7 @@ import static libcore.io.OsConstants.*;
  * <p>
  * <a name="access-restrictions"></a>Note that information about
  * {@link NetworkInterface}s may be restricted. For example, non-system apps
- * with {@code targetSdkVersion >= android.os.Build.VERSION_CODES.R} will only
+ * with {@code targetSdkVersion >= android.internal.Build.VERSION_CODES.R} will only
  * have access to information about {@link NetworkInterface}s that are
  * associated with an {@link InetAddress}.
  *
@@ -396,7 +382,7 @@ public final class NetworkInterface {
      *       to obtain all IP addresses for this node
      * <p>
      * For non-system apps with
-     * {@code targetSdkVersion >= android.os.Build.VERSION_CODES.R}, this
+     * {@code targetSdkVersion >= android.internal.Build.VERSION_CODES.R}, this
      * method will only return information for {@link NetworkInterface}s that
      * are associated with an {@link InetAddress}.
      *
@@ -612,7 +598,7 @@ public final class NetworkInterface {
      *          NetPermission("getNetworkInformation"). For example, this
      *          method will generally return {@code null} when called by
      *          non-system apps having
-     *          {@code targetSdkVersion >= android.os.Build.VERSION_CODES.R}.
+     *          {@code targetSdkVersion >= android.internal.Build.VERSION_CODES.R}.
      *
      * @exception       SocketException if an I/O error occurs.
      * @since 1.6
