@@ -89,19 +89,19 @@ std::string GetStorageType(const FieldDescriptor* descriptor) {
   }
 }
 
-std::string GetDeclarationSpace(const FieldDescriptor* descriptor) {
+std::string GetNonNullType(const FieldDescriptor* descriptor) {
   switch (GetJavaType(descriptor)) {
     case JAVATYPE_INT:
     case JAVATYPE_LONG:
     case JAVATYPE_FLOAT:
     case JAVATYPE_DOUBLE:
     case JAVATYPE_BOOLEAN:
-      return " ";
+      return GetStorageType(descriptor);
     case JAVATYPE_STRING:
     case JAVATYPE_BYTES:
     case JAVATYPE_ENUM:
     case JAVATYPE_MESSAGE:
-      return "";
+      return GetStorageType(descriptor) + " _Nonnull";
   }
 }
 
@@ -130,7 +130,7 @@ void SetCommonFieldVariables(const FieldDescriptor* descriptor,
   (*variables)["constant_name"] = FieldConstantName(descriptor);
   (*variables)["parameter_type"] = GetParameterType(descriptor);
   (*variables)["storage_type"] = GetStorageType(descriptor);
-  (*variables)["decl_space"] = GetDeclarationSpace(descriptor);
+  (*variables)["nonnull_type"] = GetNonNullType(descriptor);
   (*variables)["field_name"] = GetFieldName(descriptor);
   (*variables)["flags"] = GetFieldFlags(descriptor);
   (*variables)["field_type"] = GetFieldTypeEnumValue(descriptor);
@@ -149,10 +149,12 @@ void CollectForwardDeclarationsForFieldType(std::set<std::string>* declarations,
     declarations->insert("@class ComGoogleProtobufByteString");
   } else if (type == JAVATYPE_ENUM) {
     declarations->insert("@class " + ClassName(descriptor->enum_type()));
+    declarations->insert("@class ComGoogleProtobufDescriptors_EnumDescriptor");
     declarations->insert("J2OBJC_CLASS_DECLARATION(" +
                          ClassName(descriptor->enum_type()) + ")");
     declarations->insert(
-        "FOUNDATION_EXPORT ComGoogleProtobufDescriptors_EnumDescriptor *" +
+        "FOUNDATION_EXPORT "
+        "ComGoogleProtobufDescriptors_EnumDescriptor * _Nonnull " +
         ClassName(descriptor->enum_type()) + "_descriptor_");
   } else if (type == JAVATYPE_MESSAGE) {
     std::string classname = ClassName(descriptor->message_type());
@@ -238,7 +240,6 @@ void FieldGenerator::GenerateFieldData(io::Printer *printer) const {
   printer->Print(variables_,
       "{\n"
       "  .name = \"$field_name$\",\n"
-      "  .javaName = \"$capitalized_name$\",\n"
       "  .number = $field_number$,\n"
       "  .flags = $flags$,\n"
       "  .type = ComGoogleProtobufDescriptors_FieldDescriptor_Type_Enum_"
@@ -282,7 +283,7 @@ void FieldGenerator::GenerateStaticRefs(io::Printer *printer) const {
 SingleFieldGenerator::SingleFieldGenerator(
     const FieldDescriptor *descriptor, uint32_t *numHasBits)
   : FieldGenerator(descriptor) {
-  if (descriptor->containing_oneof() == NULL) {
+  if (descriptor->real_containing_oneof() == nullptr) {
     variables_["has_bit_index"] = SimpleItoa((*numHasBits)++);
   }
 }
@@ -290,12 +291,12 @@ SingleFieldGenerator::SingleFieldGenerator(
 void SingleFieldGenerator::GenerateFieldBuilderHeader(io::Printer* printer)
     const {
   printer->Print(variables_, "\n"
-      "- ($classname$_Builder *)set$capitalized_name$With$parameter_type$:\n"
+      "- (nonnull $classname$_Builder *)set$capitalized_name$With$parameter_type$:\n"
       "    ($storage_type$)value;\n"
-      "- ($classname$_Builder *)clear$capitalized_name$;\n");
+      "- (nonnull $classname$_Builder *)clear$capitalized_name$;\n");
   if (GetJavaType(descriptor_) == JAVATYPE_MESSAGE) {
     printer->Print(variables_,
-        "- ($classname$_Builder*)\n"
+        "- (nonnull $classname$_Builder*)\n"
         "    set$capitalized_name$With$parameter_type$_Builder:\n"
         "    ($parameter_type$_Builder *)value;\n");
   }
@@ -303,14 +304,14 @@ void SingleFieldGenerator::GenerateFieldBuilderHeader(io::Printer* printer)
 
 void SingleFieldGenerator::GenerateMessageOrBuilderProtocol(io::Printer* printer)
     const {
-  printer->Print(variables_, "\n"
-      "- (BOOL)has$capitalized_name$;\n"
-      "- ($storage_type$)get$capitalized_name$;\n"
-  );
+  printer->Print(variables_,
+                 "\n"
+                 "- (BOOL)has$capitalized_name$;\n"
+                 "- ($nonnull_type$)get$capitalized_name$;\n");
 }
 
 void SingleFieldGenerator::GenerateDeclaration(io::Printer* printer) const {
-  printer->Print(variables_, "$storage_type$$decl_space$$camelcase_name$_;\n");
+  printer->Print(variables_, "$nonnull_type$ $camelcase_name$_;\n");
 }
 
 void RepeatedFieldGenerator::CollectForwardDeclarations(
@@ -337,31 +338,32 @@ void RepeatedFieldGenerator::CollectMessageOrBuilderImports(
 void RepeatedFieldGenerator::GenerateFieldBuilderHeader(io::Printer* printer)
     const {
   printer->Print(variables_, "\n"
-      "- ($classname$_Builder *)set$capitalized_name$WithInt:(int)index\n"
+      "- (nonnull $classname$_Builder *)set$capitalized_name$WithInt:(int)index\n"
       "    with$parameter_type$:($storage_type$)value;\n"
-      "- ($classname$_Builder *)add$capitalized_name$With$parameter_type$:\n"
+      "- (nonnull $classname$_Builder *)add$capitalized_name$With$parameter_type$:\n"
       "    ($storage_type$)value;\n"
-      "- ($classname$_Builder *)addAll$capitalized_name$WithJavaLangIterable:\n"
+      "- (nonnull $classname$_Builder *)addAll$capitalized_name$WithJavaLangIterable:\n"
       "    (id<JavaLangIterable>)values;\n"
-      "- ($classname$_Builder *)clear$capitalized_name$;\n"
+      "- (nonnull $classname$_Builder *)clear$capitalized_name$;\n"
   );
   if (GetJavaType(descriptor_) == JAVATYPE_MESSAGE) {
     printer->Print(variables_,
-        "- ($classname$_Builder*)\n"
+        "- (nonnull $classname$_Builder*)\n"
         "    add$capitalized_name$With$parameter_type$_Builder:\n"
         "    ($parameter_type$_Builder *)value;\n"
-        "- ($classname$_Builder *)remove$capitalized_name$WithInt:(int)index;\n"
+        "- (nonnull $classname$_Builder *)remove$capitalized_name$WithInt:(int)index;\n"
     );
   }
 }
 
 void RepeatedFieldGenerator::GenerateMessageOrBuilderProtocol(
     io::Printer* printer) const {
-  printer->Print(variables_, "\n"
+  printer->Print(
+      variables_,
+      "\n"
       "- (jint)get$capitalized_name$Count;\n"
       "- (id<$list_type$>)get$capitalized_name$List;\n"
-      "- ($storage_type$)get$capitalized_name$WithInt:(int)index;\n"
-  );
+      "- ($nonnull_type$)get$capitalized_name$WithInt:(int)index;\n");
 }
 
 void RepeatedFieldGenerator::GenerateDeclaration(io::Printer* printer) const {
@@ -382,6 +384,7 @@ MapFieldGenerator::MapFieldGenerator(
   variables_["key_parameter_type"] = GetParameterType(key_field_);
   variables_["key_descriptor_type"] = GetFieldTypeEnumValue(key_field_);
   variables_["value_storage_type"] = GetStorageType(value_field_);
+  variables_["value_nonnull_type"] = GetNonNullType(value_field_);
   variables_["value_parameter_type"] = GetParameterType(value_field_);
   variables_["value_descriptor_type"] = GetFieldTypeEnumValue(value_field_);
   variables_["map_entry_fields_idx"] = SimpleItoa(entry_fields_idx_);
@@ -405,29 +408,31 @@ void MapFieldGenerator::CollectSourceImports(
 }
 
 void MapFieldGenerator::GenerateFieldBuilderHeader(io::Printer* printer) const {
-  printer->Print(variables_, "\n"
-      "- ($classname$_Builder *)clear$capitalized_name$;\n"
-      "- ($classname$_Builder *)remove$capitalized_name$With"
-          "$key_parameter_type$:($key_storage_type$)key;\n"
-      "- ($classname$_Builder *)put$capitalized_name$With$key_parameter_type$:"
-          "($key_storage_type$)key with$value_parameter_type$:"
-          "($value_storage_type$)value;\n"
-  );
+  printer->Print(variables_,
+                 "\n"
+                 "- (nonnull $classname$_Builder *)clear$capitalized_name$;\n"
+                 "- (nonnull $classname$_Builder *)remove$capitalized_name$With"
+                 "$key_parameter_type$:($key_storage_type$)key;\n"
+                 "- (nonnull $classname$_Builder "
+                 "*)put$capitalized_name$With$key_parameter_type$:"
+                 "($key_storage_type$)key with$value_parameter_type$:"
+                 "($value_storage_type$)value;\n");
 }
 
 void MapFieldGenerator::GenerateMessageOrBuilderProtocol(
     io::Printer* printer) const {
-  printer->Print(variables_, "\n"
+  printer->Print(
+      variables_,
+      "\n"
       "- (jint)get$capitalized_name$Count;\n"
       "- (jboolean)contains$capitalized_name$With$key_parameter_type$:"
-          "($key_storage_type$)key;\n"
-      "- (id<JavaUtilMap>)get$capitalized_name$Map;\n"
-      "- ($value_storage_type$)get$capitalized_name$OrDefaultWith"
-          "$key_parameter_type$:($key_storage_type$)key "
-          "with$value_parameter_type$:($value_storage_type$)defaultValue;\n"
-      "- ($value_storage_type$)get$capitalized_name$OrThrowWith"
-          "$key_parameter_type$:($key_storage_type$)key;\n"
-  );
+      "($key_storage_type$)key;\n"
+      "- (nonnull id<JavaUtilMap>)get$capitalized_name$Map;\n"
+      "- ($value_nonnull_type$)get$capitalized_name$OrDefaultWith"
+      "$key_parameter_type$:($key_storage_type$)key "
+      "with$value_parameter_type$:($value_storage_type$)defaultValue;\n"
+      "- ($value_nonnull_type$)get$capitalized_name$OrThrowWith"
+      "$key_parameter_type$:($key_storage_type$)key;\n");
 }
 
 void MapFieldGenerator::GenerateDeclaration(io::Printer* printer) const {
