@@ -40,6 +40,7 @@
 #include "google/protobuf/compiler/j2objc/j2objc_extension.h"
 #include "google/protobuf/compiler/j2objc/j2objc_helpers.h"
 #include "google/protobuf/compiler/j2objc/j2objc_message.h"
+#include "google/protobuf/compiler/j2objc/j2objc_message_lite.h"
 
 namespace google {
 namespace protobuf {
@@ -61,8 +62,9 @@ void AddSourceImports(std::set<std::string>& imports) {
   imports.insert("com/google/protobuf/Descriptors_PackagePrivate.h");
 }
 
-void PrintSourcePreamble(io::Printer *printer) {
-  printer->Print("\n"
+void PrintSourcePreamble(io::Printer* printer) {
+  printer->Print(
+      "\n"
       "#pragma GCC diagnostic ignored \"-Wprotocol\"\n"
       "#pragma clang diagnostic ignored \"-Wprotocol\"\n"
       "#pragma GCC diagnostic ignored \"-Wincomplete-implementation\"\n"
@@ -94,9 +96,10 @@ void PrintForwardDeclarations(const std::set<std::string>* declarations,
 
 }  // namespace
 
-FileGenerator::FileGenerator(const FileDescriptor *file)
-  : file_(file),
-    classname_(FileClassName(file)) {
+FileGenerator::FileGenerator(const FileDescriptor* file, bool enforce_lite)
+    : file_(file),
+      classname_(FileClassName(file)),
+      enforce_lite_(enforce_lite) {
   if (IsGenerateFileDirMapping()) {
     output_dir_ = FileParentDir(file);
   } else {
@@ -114,12 +117,13 @@ bool FileGenerator::Validate(std::string* error) {
   if (HasConflictingClassName(file_, classname_)) {
     error->assign(file_->name());
     error->append(
-      ": Cannot generate Java output because the file's outer class name, \"");
+        ": Cannot generate Java output because the file's outer class name, "
+        "\"");
     error->append(classname_);
     error->append(
-      "\", matches the name of one of the types declared inside it.  "
-      "Please either rename the type or use the java_outer_classname "
-      "option to specify a different outer class name for the .proto file.");
+        "\", matches the name of one of the types declared inside it.  "
+        "Please either rename the type or use the java_outer_classname "
+        "option to specify a different outer class name for the .proto file.");
     return false;
   }
 
@@ -141,10 +145,8 @@ void FileGenerator::GenerateSourceBoilerplate(io::Printer* printer) {
   GenerateBoilerplate(printer);
 }
 
-void FileGenerator::GenerateHeader(GeneratorContext* context,
-                                   std::vector<std::string>* file_list) {
+void FileGenerator::GenerateHeader(GeneratorContext* context) {
   std::string filename = GetFileName(".h");
-  file_list->push_back(filename);
 
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
@@ -154,52 +156,82 @@ void FileGenerator::GenerateHeader(GeneratorContext* context,
   std::set<std::string> headers;
   AddHeaderImports(headers);
   std::set<std::string> declarations;
-  declarations.insert("@class ComGoogleProtobufExtensionRegistry");
+  if (!enforce_lite_) {
+    declarations.insert("@class ComGoogleProtobufExtensionRegistry");
+  }
   declarations.insert("@class ComGoogleProtobufExtensionRegistryLite");
 
   if (!GenerateMultipleFiles()) {
     for (int i = 0; i < file_->message_type_count(); i++) {
-      MessageGenerator generator(file_->message_type(i));
-      generator.CollectMessageOrBuilderImports(&headers);
-      generator.CollectHeaderImports(&headers);
-      generator.CollectForwardDeclarations(&declarations);
-      generator.CollectMessageOrBuilderForwardDeclarations(&declarations);
+      MessageGenerator* generator;
+      if (enforce_lite_) {
+        generator = new MessageLiteGenerator(file_->message_type(i));
+      } else {
+        generator = new MessageGenerator(file_->message_type(i));
+      }
+      generator->CollectMessageOrBuilderImports(&headers);
+      generator->CollectHeaderImports(&headers);
+      generator->CollectForwardDeclarations(&declarations);
+      generator->CollectMessageOrBuilderForwardDeclarations(&declarations);
     }
   }
 
   PrintImports(&headers, &printer);
   PrintForwardDeclarations(&declarations, &printer);
 
-  printer.Print("\n"
+  printer.Print(
+      "\n"
+      "#pragma clang diagnostic push\n"
+      "#pragma clang diagnostic ignored \"-Wnullability-completeness\""
+      "\n"
       "@interface $classname$ : NSObject\n"
-      "\n"
-      "+ (void)registerAllExtensionsWithComGoogleProtobufExtensionRegistry:"
-          "(ComGoogleProtobufExtensionRegistry *)extensionRegistry;\n"
-      "\n"
+      "\n",
+      "classname", ClassName(file_));
+
+  if (!enforce_lite_) {
+    printer.Print(
+        "+ (void)registerAllExtensionsWithComGoogleProtobufExtensionRegistry:"
+        "(ComGoogleProtobufExtensionRegistry *)extensionRegistry;\n"
+        "\n",
+        "classname", ClassName(file_));
+  }
+
+  printer.Print(
       "+ (void)registerAllExtensionsWithComGoogleProtobufExtensionRegistryLite:"
-          "(ComGoogleProtobufExtensionRegistryLite *)extensionRegistry;\n"
+      "(ComGoogleProtobufExtensionRegistryLite *)extensionRegistry;\n"
       "\n"
-      "@end\n\n"
+      "@end\n\n",
+      "classname", ClassName(file_));
+
+  if (!enforce_lite_) {
+    printer.Print(
+        "FOUNDATION_EXPORT void $classname$_registerAllExtensionsWith"
+        "ComGoogleProtobufExtensionRegistry_("
+        "ComGoogleProtobufExtensionRegistry *extensionRegistry);\n"
+        "\n",
+        "classname", ClassName(file_));
+  }
+
+  printer.Print(
       "FOUNDATION_EXPORT void $classname$_registerAllExtensionsWith"
-          "ComGoogleProtobufExtensionRegistry_("
-          "ComGoogleProtobufExtensionRegistry *extensionRegistry);\n"
-      "\n"
-      "FOUNDATION_EXPORT void $classname$_registerAllExtensionsWith"
-          "ComGoogleProtobufExtensionRegistryLite_("
-          "ComGoogleProtobufExtensionRegistryLite *extensionRegistry);\n",
+      "ComGoogleProtobufExtensionRegistryLite_("
+      "ComGoogleProtobufExtensionRegistryLite *extensionRegistry);\n",
       "classname", ClassName(file_));
 
   if (file_->extension_count() > 0) {
-    printer.Print("\n"
+    printer.Print(
+        "\n"
         "J2OBJC_STATIC_INIT($classname$)\n",
         "classname", ClassName(file_));
   } else {
-    printer.Print("\n"
+    printer.Print(
+        "\n"
         "J2OBJC_EMPTY_STATIC_INIT($classname$)\n",
         "classname", ClassName(file_));
   }
 
-  printer.Print("\n"
+  printer.Print(
+      "\n"
       "J2OBJC_TYPE_LITERAL_HEADER($classname$)\n",
       "classname", ClassName(file_));
 
@@ -213,17 +245,23 @@ void FileGenerator::GenerateHeader(GeneratorContext* context,
     }
 
     for (int i = 0; i < file_->message_type_count(); i++) {
-      MessageGenerator generator(file_->message_type(i));
-      generator.GenerateMessageOrBuilder(&printer);
-      generator.GenerateHeader(&printer);
+      MessageGenerator* generator;
+      if (enforce_lite_) {
+        generator = new MessageLiteGenerator(file_->message_type(i));
+      } else {
+        generator = new MessageGenerator(file_->message_type(i));
+      }
+      generator->GenerateMessageOrBuilder(&printer);
+      generator->GenerateHeader(&printer);
     }
   }
+  printer.Print(
+      "\n"
+      "#pragma clang diagnostic pop\n");
 }
 
-void FileGenerator::GenerateSource(GeneratorContext* context,
-                                   std::vector<std::string>* file_list) {
+void FileGenerator::GenerateSource(GeneratorContext* context) {
   std::string filename = GetFileName(".m");
-  file_list->push_back(filename);
 
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
@@ -233,7 +271,9 @@ void FileGenerator::GenerateSource(GeneratorContext* context,
   std::set<std::string> headers;
   AddSourceImports(headers);
   headers.insert(GetFileName(".h"));
-  headers.insert("com/google/protobuf/ExtensionRegistry.h");
+  if (!enforce_lite_) {
+    headers.insert("com/google/protobuf/ExtensionRegistry.h");
+  }
   headers.insert("com/google/protobuf/ExtensionRegistryLite.h");
   if (GenerateMultipleFiles()) {
     for (int i = 0; i < file_->message_type_count(); i++) {
@@ -241,7 +281,12 @@ void FileGenerator::GenerateSource(GeneratorContext* context,
     }
   } else {
     for (int i = 0; i < file_->message_type_count(); i++) {
-      MessageGenerator(file_->message_type(i)).CollectSourceImports(&headers);
+      if (enforce_lite_) {
+        MessageLiteGenerator(file_->message_type(i))
+            .CollectSourceImports(&headers);
+      } else {
+        MessageGenerator(file_->message_type(i)).CollectSourceImports(&headers);
+      }
     }
     for (int i = 0; i < file_->enum_type_count(); i++) {
       EnumGenerator(file_->enum_type(i)).CollectSourceImports(&headers);
@@ -254,32 +299,41 @@ void FileGenerator::GenerateSource(GeneratorContext* context,
   PrintSourcePreamble(&printer);
 
   if (file_->extension_count() > 0) {
-    printer.Print(
-        "\nJ2OBJC_INITIALIZED_DEFN($classname$)\n",
-        "classname", ClassName(file_));
+    printer.Print("\nJ2OBJC_INITIALIZED_DEFN($classname$)\n", "classname",
+                  ClassName(file_));
   }
   for (int i = 0; i < file_->extension_count(); i++) {
     ExtensionGenerator(file_->extension(i)).GenerateSourceDefinition(&printer);
   }
 
-  printer.Print("\n"
+  printer.Print(
+      "\n"
       "@implementation $classname$\n"
-      "\n"
-      "+ (void)registerAllExtensionsWithComGoogleProtobufExtensionRegistry:"
-      "(ComGoogleProtobufExtensionRegistry *)extensionRegistry {\n"
-      "  $classname$_registerAllExtensionsWithComGoogleProtobuf"
-          "ExtensionRegistry_(extensionRegistry);\n"
-      "}\n"
-      "\n"
+      "\n",
+      "classname", ClassName(file_));
+
+  if (!enforce_lite_) {
+    printer.Print(
+        "+ (void)registerAllExtensionsWithComGoogleProtobufExtensionRegistry:"
+        "(ComGoogleProtobufExtensionRegistry *)extensionRegistry {\n"
+        "  $classname$_registerAllExtensionsWithComGoogleProtobuf"
+        "ExtensionRegistry_(extensionRegistry);\n"
+        "}\n"
+        "\n",
+        "classname", ClassName(file_));
+  }
+
+  printer.Print(
       "+ (void)registerAllExtensionsWithComGoogleProtobufExtensionRegistryLite:"
       "(ComGoogleProtobufExtensionRegistryLite *)extensionRegistry {\n"
       "  $classname$_registerAllExtensionsWithComGoogleProtobuf"
-          "ExtensionRegistryLite_(extensionRegistry);\n"
+      "ExtensionRegistryLite_(extensionRegistry);\n"
       "}\n",
       "classname", ClassName(file_));
 
   if (file_->extension_count() > 0) {
-    printer.Print("\n"
+    printer.Print(
+        "\n"
         "+ (void)initialize {\n"
         "  if (self == [$classname$ class]) {\n"
         "    static CGPFieldData extensionFields[] = {\n",
@@ -296,36 +350,50 @@ void FileGenerator::GenerateSource(GeneratorContext* context,
       ExtensionGenerator(file_->extension(i))
           .GenerateSourceInitializer(&printer);
     }
-    printer.Print(
-        "J2OBJC_SET_INITIALIZED($classname$)\n", "classname", ClassName(file_));
+    printer.Print("J2OBJC_SET_INITIALIZED($classname$)\n", "classname",
+                  ClassName(file_));
     printer.Outdent();
     printer.Outdent();
     printer.Print("  }\n}\n");
   }
 
-  printer.Print("\n"
+  printer.Print(
+      "\n"
       "@end\n"
       "\n"
       "J2OBJC_CLASS_TYPE_LITERAL_SOURCE($classname$)\n"
-      "\n"
+      "\n",
+      "classname", ClassName(file_));
+
+  if (!enforce_lite_) {
+    printer.Print(
+        "void $classname$_registerAllExtensionsWith"
+        "ComGoogleProtobufExtensionRegistry_("
+        "ComGoogleProtobufExtensionRegistry *extensionRegistry) {\n"
+        "  $classname$_registerAllExtensionsWith"
+        "ComGoogleProtobufExtensionRegistryLite_(extensionRegistry);\n"
+        "}\n"
+        "\n",
+        "classname", ClassName(file_));
+  }
+
+  printer.Print(
       "void $classname$_registerAllExtensionsWith"
-          "ComGoogleProtobufExtensionRegistry_("
-          "ComGoogleProtobufExtensionRegistry *extensionRegistry) {\n"
-      "  $classname$_registerAllExtensionsWith"
-          "ComGoogleProtobufExtensionRegistryLite_(extensionRegistry);\n"
-      "}\n"
-      "\n"
-      "void $classname$_registerAllExtensionsWith"
-          "ComGoogleProtobufExtensionRegistryLite_("
-          "ComGoogleProtobufExtensionRegistryLite *extensionRegistry) {\n",
+      "ComGoogleProtobufExtensionRegistryLite_("
+      "ComGoogleProtobufExtensionRegistryLite *extensionRegistry) {\n",
       "classname", ClassName(file_));
   printer.Indent();
   for (int i = 0; i < file_->extension_count(); i++) {
     ExtensionGenerator(file_->extension(i)).GenerateRegistrationCode(&printer);
   }
   for (int i = 0; i < file_->message_type_count(); i++) {
-    MessageGenerator(file_->message_type(i))
-        .GenerateExtensionRegistrationCode(&printer);
+    if (enforce_lite_) {
+      MessageLiteGenerator(file_->message_type(i))
+          .GenerateExtensionRegistrationCode(&printer);
+    } else {
+      MessageGenerator(file_->message_type(i))
+          .GenerateExtensionRegistrationCode(&printer);
+    }
   }
   printer.Outdent();
   printer.Print("}\n");
@@ -335,7 +403,11 @@ void FileGenerator::GenerateSource(GeneratorContext* context,
       EnumGenerator(file_->enum_type(i)).GenerateSource(&printer);
     }
     for (int i = 0; i < file_->message_type_count(); i++) {
-      MessageGenerator(file_->message_type(i)).GenerateSource(&printer);
+      if (enforce_lite_) {
+        MessageLiteGenerator(file_->message_type(i)).GenerateSource(&printer);
+      } else {
+        MessageGenerator(file_->message_type(i)).GenerateSource(&printer);
+      }
     }
   }
 }
@@ -348,17 +420,14 @@ std::string FileGenerator::GetFileName(std::string suffix) {
   }
 }
 
-void FileGenerator::Generate(GeneratorContext* context,
-                             std::vector<std::string>* file_list) {
-  GenerateHeader(context, file_list);
-  GenerateSource(context, file_list);
+void FileGenerator::Generate(GeneratorContext* context) {
+  GenerateHeader(context);
+  GenerateSource(context);
 }
 
 void FileGenerator::GenerateEnumHeader(GeneratorContext* context,
-                                       std::vector<std::string>* file_list,
                                        const EnumDescriptor* descriptor) {
   std::string filename = output_dir_ + descriptor->name() + ".h";
-  file_list->push_back(filename);
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -372,10 +441,8 @@ void FileGenerator::GenerateEnumHeader(GeneratorContext* context,
 }
 
 void FileGenerator::GenerateEnumSource(GeneratorContext* context,
-                                       std::vector<std::string>* file_list,
                                        const EnumDescriptor* descriptor) {
   std::string filename = output_dir_ + descriptor->name() + ".m";
-  file_list->push_back(filename);
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -392,10 +459,8 @@ void FileGenerator::GenerateEnumSource(GeneratorContext* context,
 }
 
 void FileGenerator::GenerateMessageHeader(GeneratorContext* context,
-                                          std::vector<std::string>* file_list,
                                           const Descriptor* descriptor) {
   std::string filename = output_dir_ + descriptor->name() + ".h";
-  file_list->push_back(filename);
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -415,10 +480,8 @@ void FileGenerator::GenerateMessageHeader(GeneratorContext* context,
 }
 
 void FileGenerator::GenerateMessageSource(GeneratorContext* context,
-                                          std::vector<std::string>* file_list,
                                           const Descriptor* descriptor) {
   std::string filename = output_dir_ + descriptor->name() + ".m";
-  file_list->push_back(filename);
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -435,37 +498,40 @@ void FileGenerator::GenerateMessageSource(GeneratorContext* context,
 }
 
 void FileGenerator::GenerateMessageOrBuilder(
-    GeneratorContext* context, std::vector<std::string>* file_list,
+    GeneratorContext* context,
     const Descriptor* descriptor) {
   std::string filename = output_dir_ + descriptor->name() + "OrBuilder.h";
-  file_list->push_back(filename);
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
   GenerateBoilerplate(&printer);
-  MessageGenerator generator(descriptor);
+  MessageGenerator* generator;
+  if (enforce_lite_) {
+    generator = new MessageLiteGenerator(descriptor);
+  } else {
+    generator = new MessageGenerator(descriptor);
+  }
 
   std::set<std::string> headers;
-  generator.CollectMessageOrBuilderImports(&headers);
+  generator->CollectMessageOrBuilderImports(&headers);
   PrintImports(&headers, &printer);
 
   std::set<std::string> declarations;
-  generator.CollectMessageOrBuilderForwardDeclarations(&declarations);
+  generator->CollectMessageOrBuilderForwardDeclarations(&declarations);
   PrintForwardDeclarations(&declarations, &printer);
-  generator.GenerateMessageOrBuilder(&printer);
+  generator->GenerateMessageOrBuilder(&printer);
 }
 
-void FileGenerator::GenerateSiblings(GeneratorContext* context,
-                                     std::vector<std::string>* file_list) {
+void FileGenerator::GenerateSiblings(GeneratorContext* context) {
   if (GenerateMultipleFiles()) {
     for (int i = 0; i < file_->enum_type_count(); i++) {
-      GenerateEnumHeader(context, file_list, file_->enum_type(i));
-      GenerateEnumSource(context, file_list, file_->enum_type(i));
+      GenerateEnumHeader(context, file_->enum_type(i));
+      GenerateEnumSource(context, file_->enum_type(i));
     }
     for (int i = 0; i < file_->message_type_count(); i++) {
-      GenerateMessageHeader(context, file_list, file_->message_type(i));
-      GenerateMessageSource(context, file_list, file_->message_type(i));
-      GenerateMessageOrBuilder(context, file_list, file_->message_type(i));
+      GenerateMessageHeader(context, file_->message_type(i));
+      GenerateMessageSource(context, file_->message_type(i));
+      GenerateMessageOrBuilder(context, file_->message_type(i));
     }
   }
 }
