@@ -56,8 +56,13 @@ const char *kOuterClassNameSuffix = "OuterClass";
 // j2objc-descriptor.proto.
 const int kPackagePrefixFieldNumber = 102687446;
 
+// NOLINTBEGIN(runtime/string) - Existing code design requires globals.
+static std::string globalPrefix;
+static std::string globalPostfix;
+static std::string globalFileSubExtension(".j2objc.pb");
 static std::map<std::string, std::string> prefixes;
 static std::map<std::string, std::string> wildcardPrefixes;
+// NOLINTEND(error_category)
 
 static bool generateFileDirMapping = false;
 
@@ -93,10 +98,10 @@ const std::string &FieldName(const FieldDescriptor *field) {
 }
 
 std::string StripProto(const std::string &filename) {
-  if (HasSuffixString(filename, ".protodevel")) {
-    return StripSuffixString(filename, ".protodevel");
+  if (absl::EndsWith(filename, ".protodevel")) {
+    return std::string(absl::StripSuffix(filename, ".protodevel"));
   } else {
-    return StripSuffixString(filename, ".proto");
+    return std::string(absl::StripSuffix(filename, ".proto"));
   }
 }
 
@@ -130,7 +135,7 @@ const UnknownField *FindUnknownField(const FileDescriptor *file, int field_num)
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 std::string GetPackagePrefix(const FileDescriptor *file) {
@@ -139,7 +144,7 @@ std::string GetPackagePrefix(const FileDescriptor *file) {
   const UnknownField *package_prefix_field =
       FindUnknownField(file, kPackagePrefixFieldNumber);
   if (package_prefix_field) {
-    return package_prefix_field->length_delimited();
+    return globalPrefix + package_prefix_field->length_delimited();
   }
 
   // Look for a matching prefix from the prefixes file.
@@ -156,7 +161,7 @@ std::string GetPackagePrefix(const FileDescriptor *file) {
     if (it != wildcardPrefixes.end()) {
       prefixes.insert(
           std::pair<std::string, std::string>(java_package, it->second));
-      return it->second;
+      return globalPrefix + it->second;
     }
     size_t lastDot = sub_package.find_last_of(".");
     if (lastDot == std::string::npos) {
@@ -165,12 +170,12 @@ std::string GetPackagePrefix(const FileDescriptor *file) {
     sub_package.erase(lastDot);
   }
 
-  return CapitalizeJavaPackage(java_package);
+  return globalPrefix + CapitalizeJavaPackage(java_package);
 }
 
 std::string GetJavaClassPrefix(const FileDescriptor *file,
                                const Descriptor *containing_type) {
-  if (containing_type != NULL) {
+  if (containing_type != nullptr) {
     return JavaClassName(containing_type);
   } else {
     if (file->options().java_multiple_files()) {
@@ -183,7 +188,7 @@ std::string GetJavaClassPrefix(const FileDescriptor *file,
 
 std::string GetClassPrefix(const FileDescriptor *file,
                            const Descriptor *containing_type) {
-  if (containing_type != NULL) {
+  if (containing_type != nullptr) {
     return ClassName(containing_type) + "_";
   } else {
     if (file->options().java_multiple_files()) {
@@ -305,31 +310,48 @@ std::string FileJavaPackage(const FileDescriptor *file) {
 }
 
 std::string JavaPackageToDir(std::string package_name) {
-  std::string package_dir = StringReplace(package_name, ".", "/", true);
+  std::string package_dir = absl::StrReplaceAll(package_name, {{".", "/"}});
   if (!package_dir.empty()) package_dir += "/";
   return package_dir;
 }
 
 std::string ClassName(const Descriptor *descriptor) {
-  return GetClassPrefix(descriptor->file(), descriptor->containing_type())
-      + descriptor->name();
+  return GetClassPrefix(descriptor->file(), descriptor->containing_type()) +
+         descriptor->name() + globalPostfix;
 }
 
 std::string ClassName(const EnumDescriptor *descriptor) {
-  return GetClassPrefix(descriptor->file(), descriptor->containing_type())
-      + descriptor->name();
+  return GetClassPrefix(descriptor->file(), descriptor->containing_type()) +
+         descriptor->name() + globalPostfix;
 }
 
-std::string CEnumName(const EnumDescriptor *descriptor) {
+std::string COrdinalEnumName(const EnumDescriptor *descriptor) {
   return ClassName(descriptor) + "_Enum";
 }
 
+std::string COrdinalPreprocessorName(const EnumDescriptor *descriptor) {
+  return ClassName(descriptor) + "_ORDINAL";
+}
+
+std::string CValueEnumName(const EnumDescriptor *descriptor) {
+  return ClassName(descriptor) + "_Value";
+}
+
+std::string CValuePreprocessorName(const EnumDescriptor *descriptor) {
+  return ClassName(descriptor) + "_VALUE";
+}
+
 std::string ClassName(const FileDescriptor *descriptor) {
-  return GetPackagePrefix(descriptor) + FileClassName(descriptor);
+  return GetPackagePrefix(descriptor) + FileClassName(descriptor) +
+         globalPostfix;
+}
+
+std::string EnumOrdinalName(const EnumValueDescriptor *descriptor) {
+  return COrdinalEnumName(descriptor->type()) + "_" + descriptor->name();
 }
 
 std::string EnumValueName(const EnumValueDescriptor *descriptor) {
-  return CEnumName(descriptor->type()) + "_" + descriptor->name();
+  return CValueEnumName(descriptor->type()) + "_" + descriptor->name();
 }
 
 std::string FieldConstantName(const FieldDescriptor *field) {
@@ -365,7 +387,7 @@ std::string GetHeader(const Descriptor *descriptor) {
   const FileDescriptor *file = descriptor->file();
   if (file->options().java_multiple_files()) {
     const Descriptor *containing_type = descriptor->containing_type();
-    if (containing_type != NULL) {
+    if (containing_type != nullptr) {
       return GetHeader(containing_type);
     } else {
       if (IsGenerateFileDirMapping()) {
@@ -384,7 +406,7 @@ std::string GetHeader(const EnumDescriptor *descriptor) {
   const FileDescriptor *file = descriptor->file();
   if (file->options().java_multiple_files()) {
     const Descriptor *containing_type = descriptor->containing_type();
-    if (containing_type != NULL) {
+    if (containing_type != nullptr) {
       return GetHeader(containing_type);
     } else {
       if (IsGenerateFileDirMapping()) {
@@ -470,7 +492,7 @@ static std::string HandleExtremeFloatingPoint(std::string val,
 
 // Escape C++ trigraphs by escaping question marks to \?
 static std::string EscapeTrigraphs(const std::string &to_escape) {
-  return StringReplace(to_escape, "?", "\\?", true);
+  return absl::StrReplaceAll(to_escape, {{"?", "\\?"}});
 }
 
 JavaType GetJavaType(const FieldDescriptor* field) {
@@ -563,13 +585,13 @@ std::string DefaultValue(const FieldDescriptor *field) {
         uint32_t length = ghtonl(default_string.length());
         std::string bytes((const char *)&length, sizeof(length));
         bytes.append(default_string);
-        return "\"" + CEscape(bytes) + "\"";
+        return "\"" + absl::CEscape(bytes) + "\"";
       } else {
-        return "@\"" + EscapeTrigraphs(CEscape(default_string)) + "\"";
+        return "@\"" + EscapeTrigraphs(absl::CEscape(default_string)) + "\"";
       }
     }
     case FieldDescriptor::CPPTYPE_ENUM:
-      return EnumValueName(field->default_value_enum());
+      return EnumOrdinalName(field->default_value_enum());
     case FieldDescriptor::CPPTYPE_MESSAGE:
       return "nil";
   }
@@ -629,7 +651,7 @@ std::string GetFieldOptionsData(const FieldDescriptor *descriptor) {
   if (length > 0) {
     std::string bytes((const char *)&length, sizeof(length));
     bytes.append(field_options);
-    return "\"" + CEscape(bytes) + "\"";
+    return "\"" + absl::CEscape(bytes) + "\"";
   }
   return "NULL";
 }
@@ -649,6 +671,18 @@ void ParsePrefixLine(std::string line) {
   }
 }
 
+void SetGlobalPrefix(std::string prefix) {
+  globalPrefix = std::move(prefix);
+}
+
+void SetGlobalPostfix(std::string postfix) {
+  globalPostfix = std::move(postfix);
+}
+
+void SetFileSubExtension(std::string fileSubExtension) {
+  globalFileSubExtension = std::move(fileSubExtension);
+}
+
 void ParsePrefixFile(std::string prefix_file) {
   std::ifstream in(prefix_file.c_str());
   if (in.is_open()) {
@@ -669,7 +703,7 @@ std::string MappedInputName(const FileDescriptor *file) {
 
 std::string StaticOutputFileName(const FileDescriptor *file,
                                  std::string suffix) {
-  return MappedInputName(file) + ".j2objc.pb" + suffix;
+  return MappedInputName(file) + globalFileSubExtension + suffix;
 }
 
 std::string FileDirMappingOutputName(const FileDescriptor *file) {
