@@ -49,6 +49,12 @@ namespace j2objc {
 
 namespace {
 
+// Adds the number of extension ranges in this message type to a flag vector.
+// Extensions let you declare that a range of field numbers in a message are
+// available for third-party extensions and this range is defined by the user.
+/// <param name="descriptor">describes a type of protocol message, or a
+/// particular group within a message.</param>
+/// < see cref= "protobuf/src/google/protobuf/descriptor.h" />
 std::string GetMessageFlags(const Descriptor* descriptor) {
   std::vector<std::string> flags;
   if (descriptor->extension_range_count() > 0) {
@@ -60,23 +66,30 @@ std::string GetMessageFlags(const Descriptor* descriptor) {
   return JoinFlags(flags);
 }
 
-} // namespace
+}  // namespace
 
+// Constructs a new message generator based on descriptor information such as
+// the extension range, type, etc.
+/// <param name="descriptor">describes a type of protocol message, or a
+/// particular group within a message.</param>
+/// < see cref="protobuf/src/google/protobuf/descriptor.h" />
 MessageGenerator::MessageGenerator(const Descriptor* descriptor)
-  : descriptor_(descriptor),
-  field_generators_(descriptor) {
-}
+    : descriptor_(descriptor), field_generators_(descriptor) {}
 
-MessageGenerator::~MessageGenerator() {
-}
+MessageGenerator::~MessageGenerator() = default;
 
+// Collects the forward declarations of every identifier, variable, function,
+// class, etc. contained within the descriptor. For each nested type within the
+// descriptor, this method is called again.
+/// <param name="declarations"> a set of strings to contain all of the
+/// beforehand declaration of the syntax or signature </param>
 void MessageGenerator::CollectForwardDeclarations(
     std::set<std::string>* declarations) const {
-  declarations->insert(
-      "J2OBJC_CLASS_DECLARATION(" + ClassName(descriptor_) + ")");
+  declarations->insert("J2OBJC_CLASS_DECLARATION(" + ClassName(descriptor_) +
+                       ")");
   declarations->insert("@class " + ClassName(descriptor_) + "_Builder");
-  declarations->insert(
-      "J2OBJC_CLASS_DECLARATION(" + ClassName(descriptor_) + "_Builder)");
+  declarations->insert("J2OBJC_CLASS_DECLARATION(" + ClassName(descriptor_) +
+                       "_Builder)");
   declarations->insert("@class ComGoogleProtobufDescriptors_Descriptor");
 
   for (int i = 0; i < descriptor_->field_count(); i++) {
@@ -92,6 +105,12 @@ void MessageGenerator::CollectForwardDeclarations(
   }
 }
 
+// Collects the import statements of the fields contained within the descripor.
+// If the descriptor contains space of third-party extensions, then
+// GeneratedMessage.h is added to the to the imports, otherwise
+// MessageOrBuilder.h is added.
+/// <param name="imports"> a set of strings to contain all of the import
+/// statements </param>
 void MessageGenerator::CollectMessageOrBuilderImports(
     std::set<std::string>* imports) const {
   if (descriptor_->extension_range_count() > 0) {
@@ -106,6 +125,11 @@ void MessageGenerator::CollectMessageOrBuilderImports(
   }
 }
 
+// Collects the message or builder forward declarations of all fields and oneof
+// fields within a descriptor. A "oneof" is the only one of a range of fields
+// can be set with a value in any message.
+/// <param name="declarations"> a set of strings to contain all of the
+/// beforehand declaration of the syntax or signature </param>
 void MessageGenerator::CollectMessageOrBuilderForwardDeclarations(
     std::set<std::string>* declarations) const {
   for (int i = 0; i < descriptor_->field_count(); i++) {
@@ -113,15 +137,21 @@ void MessageGenerator::CollectMessageOrBuilderForwardDeclarations(
         .CollectMessageOrBuilderForwardDeclarations(declarations);
   }
 
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i))
         .CollectMessageOrBuilderForwardDeclarations(declarations);
   }
 }
 
+// This specifically collects the header import statements of the fields
+// contained within the descripor checking both the oneof fields and nested
+// types.A "oneof" is the only one of a range of fields can be set with a value
+// in any message.
+/// <param name="imports"> a set of strings to contain all of the import
+/// statements </param>
 void MessageGenerator::CollectHeaderImports(
     std::set<std::string>* imports) const {
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i)).CollectHeaderImports(imports);
   }
 
@@ -133,6 +163,10 @@ void MessageGenerator::CollectHeaderImports(
   }
 }
 
+// Collects the source imports associated with the general fields, oneofs,
+// extensions, enums, and nested typed contained in the descriptor.
+/// <param name="imports"> a set of strings to contain all of the import
+/// statements </param>
 void MessageGenerator::CollectSourceImports(
     std::set<std::string>* imports) const {
   imports->insert("com/google/protobuf/GeneratedMessage_PackagePrivate.h");
@@ -141,7 +175,7 @@ void MessageGenerator::CollectSourceImports(
     field_generators_.get(descriptor_->field(i)).CollectSourceImports(imports);
   }
 
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i)).CollectSourceImports(imports);
   }
 
@@ -159,43 +193,58 @@ void MessageGenerator::CollectSourceImports(
   }
 }
 
+// Calls GenerateFieldHeader to print variables (classname, field type, flags,
+// parameter type, etc) associated with the field.
+// Prints the headers of th general fields, oneofs, extensions, enums, and
+// nested types contained in the descriptor by calling their respective
+// GenerateHeader.
+/// <param name="printer"> Writes text to the given output stream. It allows
+/// the caller to define a set of variables and then output some text with
+/// variable substitutions.</param>
 void MessageGenerator::GenerateHeader(io::Printer* printer) {
   std::string superclassName = "ComGoogleProtobufGeneratedMessage";
   if (descriptor_->extension_range_count() > 0) {
     superclassName = "ComGoogleProtobufGeneratedMessage_ExtendableMessage";
   }
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
+      "// in j2objc_message.cc \n"
       "@interface $classname$ : $superclassname$<$classname$OrBuilder>\n\n"
-      "+ ($classname$ *)getDefaultInstance;\n"
-      "- ($classname$ *)getDefaultInstanceForType;\n"
-      "+ ($classname$_Builder *)newBuilder OBJC_METHOD_FAMILY_NONE;\n"
-      "- ($classname$_Builder *)newBuilderForType OBJC_METHOD_FAMILY_NONE;\n"
-      "- ($classname$_Builder *)toBuilder;\n"
-      "+ ($classname$_Builder *)newBuilderWith$classname$:"
-          "($classname$ *)message OBJC_METHOD_FAMILY_NONE;\n"
-      "+ (ComGoogleProtobufDescriptors_Descriptor *)getDescriptor;\n"
+      "+ (nonnull $classname$ *)getDefaultInstance;\n"
+      "- (nonnull $classname$ *)getDefaultInstanceForType;\n"
+      "+ (nonnull $classname$_Builder *)newBuilder OBJC_METHOD_FAMILY_NONE;\n"
+      "- (nonnull $classname$_Builder *)newBuilderForType "
+      "OBJC_METHOD_FAMILY_NONE;\n"
+      "- (nonnull $classname$_Builder *)toBuilder;\n"
+      "+ (nonnull $classname$_Builder *)newBuilderWith$classname$:"
+      "($classname$ *)message OBJC_METHOD_FAMILY_NONE;\n"
+      "+ (nonnull ComGoogleProtobufDescriptors_Descriptor *)getDescriptor;\n"
       "+ ($classname$ *)parseFromWithByteArray:(IOSByteArray *)bytes;\n"
       "+ ($classname$ *)parseFromWithByteArray:(IOSByteArray *)bytes "
-          "withComGoogleProtobufExtensionRegistryLite:"
-          "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
+      "withComGoogleProtobufExtensionRegistryLite:"
+      "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
       "+ ($classname$ *)parseFromNSData:(NSData *)data;\n"
       "+ ($classname$ *)parseFromNSData:(NSData *)data registry:"
-          "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
+      "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
       "+ ($classname$ *)parseFromWithJavaIoInputStream:"
-          "(JavaIoInputStream *)input;\n"
+      "(JavaIoInputStream *)input;\n"
       "+ ($classname$ *)parseFromWithJavaIoInputStream:"
-          "(JavaIoInputStream *)bytes "
-          "withComGoogleProtobufExtensionRegistryLite:"
-          "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
+      "(JavaIoInputStream *)bytes "
+      "withComGoogleProtobufExtensionRegistryLite:"
+      "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
+      "+ ($classname$ *)parseFromWithByteString:"
+      "(ComGoogleProtobufByteString *)byteString;\n"
+      "+ (id)parseFromWithByteString:(ComGoogleProtobufByteString *)byteString "
+      "withComGoogleProtobufExtensionRegistryLite:"
+      "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
       "+ ($classname$ *)parseDelimitedFromWithJavaIoInputStream:"
-          "(JavaIoInputStream *)input;\n"
+      "(JavaIoInputStream *)input;\n"
       "+ ($classname$ *)parseDelimitedFromWithJavaIoInputStream:"
-          "(JavaIoInputStream *)bytes "
-          "withComGoogleProtobufExtensionRegistryLite:"
-          "(ComGoogleProtobufExtensionRegistryLite *)registry;\n",
-      "classname", ClassName(descriptor_),
-      "superclassname", superclassName);
+      "(JavaIoInputStream *)bytes "
+      "withComGoogleProtobufExtensionRegistryLite:"
+      "(ComGoogleProtobufExtensionRegistryLite *)registry;\n",
+      "classname", ClassName(descriptor_), "superclassname", superclassName);
 
   if (descriptor_->field_count() > 0) {
     printer->Print("\n");
@@ -208,12 +257,15 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
       "\n"
       "@end\n"
       "\n"
-      "FOUNDATION_EXPORT $classname$ *$classname$_getDefaultInstance(void);\n"
-      "FOUNDATION_EXPORT $classname$_Builder *$classname$_newBuilder(void);\n"
-      "FOUNDATION_EXPORT $classname$_Builder *$classname$_newBuilderWith"
+      "FOUNDATION_EXPORT $classname$ * _Nonnull "
+      "$classname$_getDefaultInstance(void);\n"
+      "FOUNDATION_EXPORT $classname$_Builder * _Nonnull "
+      "$classname$_newBuilder(void);\n"
+      "FOUNDATION_EXPORT $classname$_Builder * _Nonnull "
+      "$classname$_newBuilderWith"
       "$classname$_($classname$ *message);\n"
       "FOUNDATION_EXPORT ComGoogleProtobufDescriptors_Descriptor "
-      "*$classname$_getDescriptor(void);\n"
+      "* _Nonnull $classname$_getDescriptor(void);\n"
       "FOUNDATION_EXPORT $classname$ *$classname$_parseFromWithByteArray_with"
       "ComGoogleProtobufExtensionRegistryLite_(IOSByteArray *bytes, "
       "ComGoogleProtobufExtensionRegistryLite *registry);\n"
@@ -231,6 +283,16 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
       "  return $classname$_parseFromWithJavaIoInputStream_withComGoogle"
       "ProtobufExtensionRegistryLite_(input, nil);\n"
       "}\n"
+      "FOUNDATION_EXPORT $classname$ *$classname$_"
+      "parseFromWithComGoogleProtobufByteString_with"
+      "ComGoogleProtobufExtensionRegistryLite_(ComGoogleProtobufByteString "
+      "*byteString, ComGoogleProtobufExtensionRegistryLite *registry);\n"
+      "CGP_ALWAYS_INLINE inline $classname$ *$classname$_"
+      "parseFromWithComGoogleProtobufByteString_(ComGoogleProtobufByteString "
+      "*byteString) {\n"
+      "  return $classname$_parseFromWithComGoogleProtobufByteString_"
+      "withComGoogleProtobufExtensionRegistryLite_(byteString, nil);\n"
+      "}\n"
       "FOUNDATION_EXPORT $classname$ *$classname$_parseDelimitedFromWithJavaIo"
       "InputStream_withComGoogleProtobufExtensionRegistryLite_("
       "JavaIoInputStream *input, "
@@ -246,10 +308,10 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
       "J2OBJC_TYPE_LITERAL_HEADER($classname$)\n"
       "\n"
       "FOUNDATION_EXPORT ComGoogleProtobufDescriptors_Descriptor "
-      "*$classname$_descriptor_;\n",
+      "* _Nonnull $classname$_descriptor_;\n",
       "classname", ClassName(descriptor_));
 
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i)).GenerateHeader(printer);
   }
 
@@ -272,11 +334,23 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
   GenerateBuilderHeader(printer);
 }
 
+// Prints the source definition of each extension within the descriptor. Prints
+// the storage declarations of each real oneof. Prints the declation of each
+// oneof both real and synthetic. For each field not containing a oneof, it also
+// prints those declarations. Prints each map entry, the number of fields, and
+// the number of oneofs. Prints out the field data associated with each
+// extension. In addition, it prints out the source associated with each oneof,
+// enum, and nested type.
+/// <param name="printer"> Writes text to the given output stream. It allows
+/// the caller to define a set of variables and then output some text with
+/// variable substitutions.</param>
 void MessageGenerator::GenerateSource(io::Printer* printer) {
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "J2OBJC_INITIALIZED_DEFN($classname$);\n"
       "\n"
-      "ComGoogleProtobufDescriptors_Descriptor *$classname$_descriptor_;\n",
+      "ComGoogleProtobufDescriptors_Descriptor * _Nonnull "
+      "$classname$_descriptor_;\n",
       "classname", ClassName(descriptor_));
 
   for (int i = 0; i < descriptor_->extension_count(); i++) {
@@ -284,34 +358,40 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
         .GenerateSourceDefinition(printer);
   }
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "@implementation $classname$\n",
       "classname", ClassName(descriptor_));
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "typedef struct $classname$_Storage {\n"
       "  uint32_t hasBits[$num_has_bytes$];\n",
-      "classname", ClassName(descriptor_),
-      "num_has_bytes", SimpleItoa((field_generators_.numHasBits() + 31) / 32));
+      "classname", ClassName(descriptor_), "num_has_bytes",
+      SimpleItoa((field_generators_.numHasBits() + 31) / 32));
 
   printer->Indent();
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i))
         .GenerateStorageDeclaration(printer);
   }
   for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
     const OneofDescriptor* oneof = descriptor_->oneof_decl(i);
-    printer->Print("union {\n");
-    printer->Indent();
-    for (int j = 0; j < oneof->field_count(); j++) {
-      field_generators_.get(oneof->field(j)).GenerateDeclaration(printer);
+    if (oneof->field_count() == 1) {
+      field_generators_.get(oneof->field(0)).GenerateDeclaration(printer);
+    } else {
+      printer->Print("union {\n");
+      printer->Indent();
+      for (int j = 0; j < oneof->field_count(); j++) {
+        field_generators_.get(oneof->field(j)).GenerateDeclaration(printer);
+      }
+      printer->Outdent();
+      printer->Print("};\n");
     }
-    printer->Outdent();
-    printer->Print("};\n");
   }
   for (int i = 0; i < descriptor_->field_count(); i++) {
     const FieldDescriptor* field = descriptor_->field(i);
-    if (field->containing_oneof() == NULL) {
+    if (field->containing_oneof() == nullptr) {
       field_generators_.get(field).GenerateDeclaration(printer);
     }
   }
@@ -320,24 +400,25 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   printer->Print(
       "} $classname$_Storage;\n"
       "\n"
-      "+ (ComGoogleProtobufDescriptors_Descriptor *)getDescriptor {\n"
+      "+ (nonnull ComGoogleProtobufDescriptors_Descriptor *)getDescriptor {\n"
       "  return $classname$_descriptor_;\n"
       "}\n",
       "classname", ClassName(descriptor_));
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "// Minimal metadata for runtime access to Java class name.\n"
       "+ (const J2ObjcClassInfo *)__metadata {\n"
       "  static const J2ObjcClassInfo _$classname$ = { \"$simplename$\", "
-          "\"$packagename$\", NULL, NULL, NULL, 7, 0x1, 0, 0, -1, -1, -1, "
-          "-1, -1 };\n"
+      "\"$packagename$\", NULL, NULL, NULL, 7, 0x1, 0, 0, -1, -1, -1, "
+      "-1, -1 };\n"
       "  return &_$classname$;\n"
       "}\n",
-      "classname", ClassName(descriptor_),
-      "simplename", descriptor_->name(),
+      "classname", ClassName(descriptor_), "simplename", descriptor_->name(),
       "packagename", FileJavaPackage(descriptor_->file()));
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "+ (void)initialize {\n"
       "  if (self == [$classname$ class]) {\n",
       "classname", ClassName(descriptor_));
@@ -348,10 +429,10 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   // in an access of this descriptor during its class initialization.
   printer->Print(
       "$classname$_descriptor_ = CGPInitDescriptor(self, "
-          "[$classname$_Builder class], $flags$, "
-          "sizeof($classname$_Storage));\n",
-      "classname", ClassName(descriptor_),
-      "flags", GetMessageFlags(descriptor_));
+      "[$classname$_Builder class], $flags$, "
+      "sizeof($classname$_Storage));\n",
+      "classname", ClassName(descriptor_), "flags",
+      GetMessageFlags(descriptor_));
   if (field_generators_.numMapFields() > 0) {
     printer->Print("static CGPFieldData mapEntryFields[] = {\n");
     printer->Indent();
@@ -369,10 +450,10 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   }
   printer->Outdent();
   printer->Print("};\n");
-  if (descriptor_->oneof_decl_count() > 0) {
+  if (descriptor_->real_oneof_decl_count() > 0) {
     printer->Print("static CGPOneofData oneofs[] = {\n");
     printer->Indent();
-    for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+    for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
       OneofGenerator(descriptor_->oneof_decl(i)).GenerateOneofData(printer);
     }
     printer->Outdent();
@@ -380,11 +461,11 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   }
   printer->Print(
       "CGPInitFields($classname$_descriptor_, $fieldcount$, fields, "
-          "$oneofcount$, $oneofdata$);\n",
-      "classname", ClassName(descriptor_),
-      "fieldcount", SimpleItoa(descriptor_->field_count()),
-      "oneofcount", SimpleItoa(descriptor_->oneof_decl_count()),
-      "oneofdata", descriptor_->oneof_decl_count() > 0 ? "oneofs" : "NULL");
+      "$oneofcount$, $oneofdata$);\n",
+      "classname", ClassName(descriptor_), "fieldcount",
+      SimpleItoa(descriptor_->field_count()), "oneofcount",
+      SimpleItoa(descriptor_->real_oneof_decl_count()), "oneofdata",
+      descriptor_->real_oneof_decl_count() > 0 ? "oneofs" : "NULL");
 
   if (descriptor_->extension_count() > 0) {
     printer->Print("static CGPFieldData extensionFields[] = {\n");
@@ -399,9 +480,8 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
           .GenerateSourceInitializer(printer);
     }
   }
-  printer->Print(
-      "J2OBJC_SET_INITIALIZED($classname$)\n",
-      "classname", ClassName(descriptor_));
+  printer->Print("J2OBJC_SET_INITIALIZED($classname$)\n", "classname",
+                 ClassName(descriptor_));
   printer->Outdent();
   printer->Outdent();
   printer->Print("  }\n}\n");
@@ -412,19 +492,19 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
       "\n"
       "J2OBJC_CLASS_TYPE_LITERAL_SOURCE($classname$)\n"
       "\n"
-      "$classname$ *$classname$_getDefaultInstance(void) {\n"
+      "$classname$ * _Nonnull $classname$_getDefaultInstance(void) {\n"
       "  $classname$_initialize();\n"
       "  return AUTORELEASE("
       "($classname$ *)CGPNewMessage($classname$_descriptor_));\n"
       "}\n"
       "\n"
-      "$classname$_Builder *$classname$_newBuilder(void) {\n"
+      "$classname$_Builder * _Nonnull $classname$_newBuilder(void) {\n"
       "  $classname$_initialize();\n"
       "  return AUTORELEASE("
       "($classname$_Builder *)CGPNewBuilder($classname$_descriptor_));\n"
       "}\n"
       "\n"
-      "$classname$_Builder *$classname$_newBuilderWith$classname$_("
+      "$classname$_Builder *_Nonnull $classname$_newBuilderWith$classname$_("
       "$classname$ *message) {\n"
       "  $classname$_initialize();\n"
       "  return ($classname$_Builder *)CGPBuilderFromPrototype("
@@ -432,7 +512,7 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
       "}\n"
       "\n"
       "ComGoogleProtobufDescriptors_Descriptor "
-      "*$classname$_getDescriptor(void) {\n"
+      "* _Nonnull $classname$_getDescriptor(void) {\n"
       "  $classname$_initialize();\n"
       "  return $classname$_descriptor_;\n"
       "}\n"
@@ -452,6 +532,13 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
       "  return ($classname$ *)CGPParseFromInputStream("
       "$classname$_descriptor_, input, registry);\n"
       "}\n"
+      "$classname$ *$classname$_parseFromWithComGoogleProtobufByteString_with"
+      "ComGoogleProtobufExtensionRegistryLite_(ComGoogleProtobufByteString "
+      "*input, ComGoogleProtobufExtensionRegistryLite *registry) {\n"
+      "  $classname$_initialize();\n"
+      "  return ($classname$ *)CGPParseFromByteString("
+      "$classname$_descriptor_, input, registry);\n"
+      "}\n"
       "$classname$ *$classname$_parseDelimitedFromWithJavaIoInputStream_with"
       "ComGoogleProtobufExtensionRegistryLite_(JavaIoInputStream *input,"
       " ComGoogleProtobufExtensionRegistryLite *registry) {\n"
@@ -461,7 +548,7 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
       "}\n",
       "classname", ClassName(descriptor_));
 
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i)).GenerateSource(printer);
   }
 
@@ -477,26 +564,32 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   GenerateBuilderSource(printer);
 }
 
+// Prints the builder associated with the descriptor as a whole.
+// Prints the builder header associated with each field contained within the
+// descriptor in addition it will specify if the message is a java type message.
+/// <param name="printer"> Writes text to the given output stream. It allows
+/// the caller to define a set of variables and then output some text with
+/// variable substitutions.</param>
 void MessageGenerator::GenerateBuilderHeader(io::Printer* printer) {
   std::string superclassName = "ComGoogleProtobufGeneratedMessage_Builder";
   if (descriptor_->extension_range_count() > 0) {
     superclassName = "ComGoogleProtobufGeneratedMessage_ExtendableBuilder";
   }
 
-  printer->Print("\n"
-      "@interface $classname$_Builder : "
-          "$superclassname$<$classname$OrBuilder>\n"
+  printer->Print(
       "\n"
-      "- ($classname$ *)getDefaultInstanceForType;\n"
-      "- ($classname$_Builder *)mergeFromWith$classname$:"
-          "($classname$ *)message;\n"
-      "- ($classname$_Builder *)mergeFromWithComGoogleProtobufMessage:"
-          "(id<ComGoogleProtobufMessage>)message;\n"
-      "- ($classname$ *)build;\n"
-      "- ($classname$ *)buildPartial;\n"
-      "+ (ComGoogleProtobufDescriptors_Descriptor *)getDescriptor;\n",
-      "classname", ClassName(descriptor_),
-      "superclassname", superclassName);
+      "@interface $classname$_Builder : "
+      "$superclassname$<$classname$OrBuilder>\n"
+      "\n"
+      "- (nonnull $classname$ *)getDefaultInstanceForType;\n"
+      "- (nonnull $classname$_Builder *)mergeFromWith$classname$:"
+      "($classname$ *)message;\n"
+      "- (nonnull $classname$_Builder *)mergeFromWithComGoogleProtobufMessage:"
+      "(id<ComGoogleProtobufMessage>)message;\n"
+      "- (nonnull $classname$ *)build;\n"
+      "- (nonnull $classname$ *)buildPartial;\n"
+      "+ (nonnull ComGoogleProtobufDescriptors_Descriptor *)getDescriptor;\n",
+      "classname", ClassName(descriptor_), "superclassname", superclassName);
 
   for (int i = 0; i < descriptor_->field_count(); i++) {
     field_generators_.get(descriptor_->field(i))
@@ -507,7 +600,7 @@ void MessageGenerator::GenerateBuilderHeader(io::Printer* printer) {
       "\n"
       "@end\n\n"
       "FOUNDATION_EXPORT ComGoogleProtobufDescriptors_Descriptor "
-      "*$classname$_Builder_getDescriptor(void);\n"
+      "* _Nonnull $classname$_Builder_getDescriptor(void);\n"
       "\n"
       "J2OBJC_EMPTY_STATIC_INIT($classname$_Builder)\n"
       "\n"
@@ -515,6 +608,10 @@ void MessageGenerator::GenerateBuilderHeader(io::Printer* printer) {
       "classname", ClassName(descriptor_));
 }
 
+// Prints the builder source associated with the descriptor.
+/// <param name="printer"> Writes text to the given output stream. It allows
+/// the caller to define a set of variables and then output some text with
+/// variable substitutions.</param>
 void MessageGenerator::GenerateBuilderSource(io::Printer* printer) {
   printer->Print(
       "\n"
@@ -530,13 +627,18 @@ void MessageGenerator::GenerateBuilderSource(io::Printer* printer) {
       "J2OBJC_INTERFACE_TYPE_LITERAL_SOURCE($classname$OrBuilder)\n"
       "\n"
       "ComGoogleProtobufDescriptors_Descriptor "
-      "*$classname$_Builder_getDescriptor(void) {\n"
+      "* _Nonnull $classname$_Builder_getDescriptor(void) {\n"
       "  $classname$_initialize();\n"
       "  return $classname$_descriptor_;\n"
       "}\n",
       "classname", ClassName(descriptor_));
 }
 
+// Prints the name of the protocol directory along with name/class name
+// associated with each field and oneof
+/// <param name="printer"> Writes text to the given output stream. It allows
+/// the caller to define a set of variables and then output some text with
+/// variable substitutions.</param>
 void MessageGenerator::GenerateMessageOrBuilder(io::Printer* printer) {
   std::string protocolName = "ComGoogleProtobufMessageOrBuilder";
   if (descriptor_->extension_range_count() > 0) {
@@ -544,22 +646,23 @@ void MessageGenerator::GenerateMessageOrBuilder(io::Printer* printer) {
         "ComGoogleProtobufGeneratedMessage_ExtendableMessageOrBuilder";
   }
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "@protocol $classname$OrBuilder < $protocolname$ >\n",
-      "classname", ClassName(descriptor_),
-      "protocolname", protocolName);
+      "classname", ClassName(descriptor_), "protocolname", protocolName);
 
   for (int i = 0; i < descriptor_->field_count(); i++) {
     field_generators_.get(descriptor_->field(i))
         .GenerateMessageOrBuilderProtocol(printer);
   }
 
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
     OneofGenerator(descriptor_->oneof_decl(i))
         .GenerateMessageOrBuilder(printer);
   }
 
-  printer->Print("\n"
+  printer->Print(
+      "\n"
       "@end\n"
       "\n"
       "J2OBJC_EMPTY_STATIC_INIT($classname$OrBuilder)\n"
@@ -568,6 +671,11 @@ void MessageGenerator::GenerateMessageOrBuilder(io::Printer* printer) {
       "classname", ClassName(descriptor_));
 }
 
+// Prints the registration codes associated with each extension and nested
+// type's  extension in the descriptor
+/// <param name="printer"> Writes text to the given output stream. It allows
+/// the caller to define a set of variables and then output some text with
+/// variable substitutions.</param>
 void MessageGenerator::GenerateExtensionRegistrationCode(io::Printer* printer) {
   for (int i = 0; i < descriptor_->extension_count(); i++) {
     ExtensionGenerator(descriptor_->extension(i))
