@@ -38,7 +38,7 @@
 #import "com/google/protobuf/DescriptorProtos.h"
 #import "com/google/protobuf/GeneratedMessage_PackagePrivate.h"
 #import "com/google/protobuf/ProtocolMessageEnum.h"
-#import "com/google/protobuf/WireFormat.h"
+#import "com/google/protobuf/WireFormat_PackagePrivate.h"
 #import "java/lang/Boolean.h"
 #import "java/lang/Double.h"
 #import "java/lang/Enum.h"
@@ -137,7 +137,7 @@ CGPDescriptor *NewMapEntryDescriptor(CGPFieldData *fieldData) {
 
 CGPEnumDescriptor *CGPInitializeEnumType(
     Class enumClass, jint valuesCount, JavaLangEnum<ComGoogleProtobufProtocolMessageEnum> *values[],
-    NSString **names, jint *intValues) {
+    NSString **names, jint *intValues, jboolean is_closed) {
   Ivar valueIvar = class_getInstanceVariable(enumClass, "value_");
   ptrdiff_t valueOffset = ivar_getOffset(valueIvar);
 
@@ -175,7 +175,9 @@ CGPEnumDescriptor *CGPInitializeEnumType(
   // Construct the enum descriptor.
   CGPEnumDescriptor *enumDesc =
       objc_constructInstance([CGPEnumDescriptor class], (void *)enumDescPtr);
-  return [enumDesc initWithValueOffset:valueOffset retainedValues:valuesArray];
+  return [enumDesc initWithValueOffset:valueOffset
+                        retainedValues:valuesArray
+                             is_closed:is_closed];
 }
 
 void CGPInitializeOneofCaseEnum(
@@ -358,7 +360,6 @@ static void CGPFieldFixDefaultValue(CGPFieldDescriptor *descriptor) {
     data_ = data;
     tag_ = TagFromData(data);
     javaType_ = [GetTypeObj(data->type)->javaType_ ordinal];
-    fieldOptions_ = InitFieldOptions(data->optionsData);
     containingType_ = containingType;
     CGPFieldFixDefaultValue(self);
   }
@@ -425,6 +426,9 @@ static void CGPFieldFixDefaultValue(CGPFieldDescriptor *descriptor) {
 - (ComGoogleProtobufDescriptorProtos_FieldOptions *)getOptions {
   if (fieldOptions_ != nil) {
     return fieldOptions_;
+  } else if (data_->optionsData != NULL) {
+    fieldOptions_ = InitFieldOptions(data_->optionsData);
+    return fieldOptions_;
   } else {
     return [ComGoogleProtobufDescriptorProtos_FieldOptions getDefaultInstance];
   }
@@ -451,22 +455,27 @@ id CGPFieldGetDefaultValue(CGPFieldDescriptor *field) {
 
 CGPEnumValueDescriptor *CGPEnumValueDescriptorFromInt(CGPEnumDescriptor *enumType, jint value) {
   NSUInteger count = enumType->values_->size_;
+  NSUInteger numValues = enumType->is_closed_ ? count : count - 1;  // Skip the UNRECOGNIZED value.
   CGPEnumValueDescriptor **valuesBuf = enumType->values_->buffer_;
-  for (NSUInteger i = 0; i < count; i++) {
+  for (NSUInteger i = 0; i < numValues; i++) {
     CGPEnumValueDescriptor *valueDescriptor = valuesBuf[i];
     if (valueDescriptor->number_ == value) {
       return valueDescriptor;
     }
   }
-  return nil;
+  // If proto3 (not closed), the UNRECOGNIZED value is the last values element.
+  return enumType->is_closed_ ? nil : valuesBuf[count - 1];
 }
 
 @implementation ComGoogleProtobufDescriptors_EnumDescriptor
 
-- (instancetype)initWithValueOffset:(ptrdiff_t)valueOffset retainedValues:(IOSObjectArray *)values {
+- (instancetype)initWithValueOffset:(ptrdiff_t)valueOffset
+                     retainedValues:(IOSObjectArray *)values
+                          is_closed:(jboolean)is_closed {
   if (self = [super init]) {
     valueOffset_ = valueOffset;
     values_ = values; // Already retained.
+    is_closed_ = is_closed;
   }
   return self;
 }
