@@ -232,6 +232,24 @@ public class CompatibilityTest extends ProtobufTest {
     assertFalse(builder.hasExtension(Typical.myPrimitiveExtension));
   }
 
+  public void testSetRepeatedExtension() throws Exception {
+    TypicalData.Builder builder = TypicalData.newBuilder();
+    builder.addExtension(Typical.myRepeatedPrimitiveExtension, 111);
+    builder.addExtension(Typical.myRepeatedPrimitiveExtension, 222);
+    assertEquals(2, builder.getExtensionCount(Typical.myRepeatedPrimitiveExtension));
+    assertEquals(
+        111, ((Integer) builder.getExtension(Typical.myRepeatedPrimitiveExtension, 0)).intValue());
+    assertEquals(
+        222, ((Integer) builder.getExtension(Typical.myRepeatedPrimitiveExtension, 1)).intValue());
+
+    builder.setExtension(Typical.myRepeatedPrimitiveExtension, 0, 333);
+    assertEquals(2, builder.getExtensionCount(Typical.myRepeatedPrimitiveExtension));
+    assertEquals(
+        333, ((Integer) builder.getExtension(Typical.myRepeatedPrimitiveExtension, 0)).intValue());
+    assertEquals(
+        222, ((Integer) builder.getExtension(Typical.myRepeatedPrimitiveExtension, 1)).intValue());
+  }
+
   public void testClearRepeatedField() throws Exception {
     TypicalData.Builder builder = TypicalData.newBuilder()
         .addRepeatedInt32(1)
@@ -510,7 +528,7 @@ public class CompatibilityTest extends ProtobufTest {
     assertFalse(fieldDescriptor.isExtension());
 
     ExtensionRegistry.ExtensionInfo extensionInfo =
-        registry.findExtensionByNumber(descriptor, 1000);
+        registry.findImmutableExtensionByNumber(descriptor, 1000);
     assertNotNull(extensionInfo);
 
     FieldDescriptor extensionFieldDescriptor = extensionInfo.descriptor;
@@ -527,7 +545,7 @@ public class CompatibilityTest extends ProtobufTest {
     assertEquals(100, data.getMyMessageInt());
 
     // Primitive extension
-    extensionInfo = registry.findExtensionByNumber(descriptor, 1001);
+    extensionInfo = registry.findImmutableExtensionByNumber(descriptor, 1001);
     assertNotNull(extensionInfo);
 
     extensionFieldDescriptor = extensionInfo.descriptor;
@@ -632,18 +650,18 @@ public class CompatibilityTest extends ProtobufTest {
     }
 
     TypicalData.Builder dataBuilder = TypicalData.newBuilder();
-    dataBuilder.setField(fields[1], new Integer(42));
+    dataBuilder.setField(fields[1], Integer.valueOf(42));
     dataBuilder.setField(fields[2], ByteString.copyFromUtf8("foo"));
     dataBuilder.setField(fields[3], TypicalData.EnumType.VALUE9.getValueDescriptor());
     dataBuilder.setField(fields[11], TypicalDataMessage.newBuilder().build());
     dataBuilder.setField(fields[12], Boolean.TRUE);
-    dataBuilder.setField(fields[13], new Float(43.8));
-    dataBuilder.setField(fields[14], new Double(44.5));
+    dataBuilder.setField(fields[13], Float.valueOf((float) 43.8));
+    dataBuilder.setField(fields[14], Double.valueOf(44.5));
     dataBuilder.setField(fields[15], "bar");
-    dataBuilder.setField(fields[16], new Integer(24));
-    dataBuilder.setField(fields[17], new Long(4422));
-    dataBuilder.setField(fields[18], new Long(2244));
-    dataBuilder.addRepeatedField(fields[4], new Integer(72));
+    dataBuilder.setField(fields[16], Integer.valueOf(24));
+    dataBuilder.setField(fields[17], Long.valueOf(4422));
+    dataBuilder.setField(fields[18], Long.valueOf(2244));
+    dataBuilder.addRepeatedField(fields[4], Integer.valueOf(72));
     dataBuilder.addRepeatedField(fields[8], "aaa");
     dataBuilder.addRepeatedField(fields[8], "bbb");
     dataBuilder.setRepeatedField(fields[8], 1, "ccc");
@@ -661,8 +679,8 @@ public class CompatibilityTest extends ProtobufTest {
     assertEquals("foo", new String(data.getMyBytes().toByteArray()));
     assertEquals(TypicalData.EnumType.VALUE9, data.getMyEnumType());
     assertTrue(data.getMyBool());
-    assertEquals(new Float(43.8), data.getMyFloat());
-    assertEquals(new Double(44.5), data.getMyDouble());
+    assertEquals(Float.valueOf((float) 43.8), data.getMyFloat());
+    assertEquals(Double.valueOf(44.5), data.getMyDouble());
     assertEquals("bar", data.getMyString());
     assertEquals(24, data.getMyUint());
     assertEquals(4422, data.getMyLong());
@@ -1092,7 +1110,7 @@ public class CompatibilityTest extends ProtobufTest {
     MessageLite.Builder builder = TypicalData.newBuilder();
     MessageLite message = builder.build();
     assertTrue(message instanceof MessageLite);
-    message.toByteString();
+    ByteString unused = message.toByteString();
   }
 
   public void testMutatingBuilderDoesntMutateMessage() throws Exception {
@@ -1204,23 +1222,42 @@ public class CompatibilityTest extends ProtobufTest {
     assertEquals(0, messageLite.getSerializedSize());
   }
 
-  /**
-   * The returned list must be able to see edits to the internal list. However
-   * when the internal list is cleared the returned list maintains a view of the
-   * elements as they where before the clear.
-   */
-  public void testGetRepeatedList() throws Exception {
-    TypicalData.Builder builder = TypicalData.newBuilder()
-        .addRepeatedInt32(1)
-        .addRepeatedInt32(2)
-        .addRepeatedInt32(3);
-    List<Integer> list = builder.getRepeatedInt32List();
-    assertEquals(3, list.size());
-    builder.setRepeatedInt32(1, 4);
-    assertEquals(4, list.get(1).intValue());
-    builder.clearRepeatedInt32();
-    assertEquals(3, list.size());
-  }
+  // TODO(b/282022385): reenable these two tests when BlazeTests and BlazeTestsIos are both
+  // passing.
+  // /**
+  //  * The list retrieved from list getter should be just a snapshot copy of the internal list and
+  //  * thus should not reflect the changes to the field that are via builder accessors.
+  //  */
+  // public void testGetRepeatedList() throws Exception {
+  //   TypicalData.Builder builder = TypicalData.newBuilder()
+  //       .addRepeatedInt32(1)
+  //       .addRepeatedInt32(2)
+  //       .addRepeatedInt32(3);
+  //   List<Integer> list = builder.getRepeatedInt32List();
+  //   assertThat(list).hasSize(3);
+  //   builder.setRepeatedInt32(1, 4);
+
+  //   // `list` is a copy or a snapshot of the message, so it should not modified when setting
+  //   // values with builders.
+  //   assertEquals(2, list.get(1).intValue());
+  //   builder.clearRepeatedInt32();
+  //   assertThat(list).hasSize(3);
+  // }
+
+  // /** The same as `testGetRepeatedList` but for repeated string fields. */
+  // public void testGetRepeatedStringList() throws Exception {
+  //   TypicalData.Builder builder =
+  //       TypicalData.newBuilder()
+  //           .addRepeatedString("1")
+  //           .addRepeatedString("2")
+  //           .addRepeatedString("3");
+  //   ProtocolStringList list = builder.getRepeatedStringList();
+  //   assertThat(list).hasSize(3);
+  //   builder.setRepeatedString(1, "4");
+  //   assertEquals("2", list.get(1));
+  //   builder.clearRepeatedString();
+  //   assertThat(list).hasSize(3);
+  // }
 
   public void testGetByteArray() throws Exception {
     // Make sure it compiles with the MessageLite type.
@@ -1237,7 +1274,7 @@ public class CompatibilityTest extends ProtobufTest {
     // Extension added to registry should be visible in registry2.
     Descriptor descriptor = TypicalData.getDescriptor();
     ExtensionRegistry.ExtensionInfo extensionInfo =
-        registry2.findExtensionByNumber(descriptor, 1001);
+        registry2.findImmutableExtensionByNumber(descriptor, 1001);
     assertNotNull(extensionInfo);
 
     ExtensionRegistryLite registryLite = ExtensionRegistryLite.newInstance();
