@@ -18,6 +18,7 @@ import static com.google.common.io.FileWriteMode.APPEND;
 import static java.nio.charset.StandardCharsets.UTF_8;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Ascii;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Splitter;
 import com.google.common.collect.ImmutableSet;
@@ -60,11 +61,16 @@ import java.util.regex.Pattern;
  *
  * @author Tom Ball
  */
+@SuppressWarnings("SystemExitOutsideMain")
 public class Options {
 
   private List<String> processorPathEntries = new ArrayList<>();
   private OutputLanguageOption language = OutputLanguageOption.OBJECTIVE_C;
   private MemoryManagementOption memoryManagementOption = null;
+  private boolean strictFieldAssign = false;
+  private boolean strictFieldLoad = false;
+  private boolean retainAutoreleaseReturns = false;
+  private boolean arcAutoreleaseReturns = false;
   private EmitLineDirectivesOption emitLineDirectives = EmitLineDirectivesOption.NONE;
   private boolean warningsAsErrors = false;
   private boolean deprecatedDeclarations = false;
@@ -84,10 +90,11 @@ public class Options {
   private String processors = null;
   private boolean disallowInheritedConstructors = true;
   private boolean nullability = false;
-  private boolean defaultNonnull = false;
+  private boolean swiftEnums = true;
+  private boolean nullMarked = false;
   private TimingLevel timingLevel = TimingLevel.NONE;
   private boolean dumpAST = false;
-  private String lintArgument = null;
+  private String lintArgument = "-Xlint:none"; // Disable all lint warnings by default.
   private boolean reportJavadocWarnings = false;
   private boolean translateBootclasspath = false;
   private boolean translateClassfiles = false;
@@ -97,6 +104,12 @@ public class Options {
   private boolean emitKytheMappings = false;
   private boolean emitSourceHeaders = true;
   private boolean injectLogSites = false;
+  private boolean allVersions = false;
+  private boolean asObjCGenericDecl = false;
+  private boolean ignoreJarWarnings = false;
+  private boolean linkSourcePathHeaders = false;
+  private boolean javacWarnings = true;
+  private boolean stripReflectionErrors = false;
 
   private Mappings mappings = new Mappings();
   private FileUtil fileUtil = new FileUtil();
@@ -284,7 +297,8 @@ public class Options {
   /**
    * Set all log handlers in this package with a common level.
    */
-  private void setLogLevel(Level level) {
+  @VisibleForTesting
+  void setLogLevel(Level level) {
     logger.setLevel(level);
     logLevelSet = true;
   }
@@ -320,12 +334,17 @@ public class Options {
   private class ArgProcessor {
 
     private final List<String> sourceFiles = new ArrayList<>();
+    private boolean printArgs = false;
 
     private void processArgs(String[] args) throws IOException {
       Iterator<String> iter = Arrays.asList(args).iterator();
       while (iter.hasNext()) {
         processArg(iter);
       }
+      if (printArgs) {
+        System.err.print("j2objc ");
+        System.err.println(String.join(" ", args));
+      }
     }
 
     private void processArgsFile(String filename) throws IOException {
@@ -399,6 +418,16 @@ public class Options {
         fileUtil.setHeaderOutputDirectory(new File(getArgValue(args, arg)));
       } else if (arg.equals("-use-arc")) {
         checkMemoryManagementOption(MemoryManagementOption.ARC);
+      } else if (arg.equals("-Xstrict-field-assign")) {
+        strictFieldAssign = true;
+      } else if (arg.equals("-Xstrict-field-load")) {
+        strictFieldAssign = true;
+        strictFieldLoad = true;
+      } else if (arg.equals("-Xretain-autorelease-returns")) {
+        retainAutoreleaseReturns = true;
+      } else if (arg.equals("-Xarc-autorelease-returns")) {
+        retainAutoreleaseReturns = true;
+        arcAutoreleaseReturns = true;
       } else if (arg.equals("-g")) {
         emitLineDirectives = EmitLineDirectivesOption.NORMAL;
       } else if (arg.equals("-g:none")) {
@@ -416,7 +445,7 @@ public class Options {
       } else if (arg.startsWith(TIMING_INFO_ARG + ':')) {
         String timingArg = arg.substring(TIMING_INFO_ARG.length() + 1);
         try {
-          timingLevel = TimingLevel.valueOf(timingArg.toUpperCase());
+          timingLevel = TimingLevel.valueOf(Ascii.toUpperCase(timingArg));
         } catch (IllegalArgumentException e) {
           usage("invalid --timing-info argument");
         }
@@ -428,6 +457,10 @@ public class Options {
         // TODO(tball): remove flag when all client builds stop using it.
       } else if (arg.equals("-Xno-jsni-warnings")) {
         jsniWarnings = false;
+      } else if (arg.equals("-Xignore-jar-warnings")) {
+        ignoreJarWarnings = true;
+      } else if (arg.equals("-Xlink-source-path-headers")) {
+        linkSourcePathHeaders = true;
       } else if (arg.equals("-encoding")) {
         try {
           fileUtil.setFileEncoding(getArgValue(args, arg));
@@ -438,8 +471,40 @@ public class Options {
         stripGwtIncompatible = true;
       } else if (arg.equals("--strip-reflection")) {
         includedMetadata = EnumSet.of(MetadataSupport.ENUM_CONSTANTS);
+      } else if (arg.equals("-Xstrip-reflection-errors:")) {
+        String subArg = arg.substring(arg.indexOf(':') + 1);
+        switch (subArg) {
+          case "true":
+            {
+              setStripReflectionErrors(true);
+              break;
+            }
+          case "false":
+            {
+              break;
+            }
+          default:
+            {
+              usage("invalid -Xstrip-reflection-warnings argument: " + subArg);
+            }
+        }
       } else if (arg.equals("-Xstrip-enum-constants")) {
         includedMetadata.remove(MetadataSupport.ENUM_CONSTANTS);
+      } else if (arg.startsWith("-Xjavac-warnings:")) {
+        String subArg = arg.substring(arg.indexOf(':') + 1);
+        switch (subArg) {
+          case "true": {
+            javacWarnings = true;
+            break;
+          }
+          case "false": {
+            javacWarnings = false;
+            break;
+          }
+          default: {
+            usage("invalid -Xjavac-warnings argument: " + subArg);
+          }
+        }
       } else if (arg.startsWith("--reflection:")) {
         includedMetadata.remove(MetadataSupport.FULL);
         String[] subArgs = arg.substring(arg.indexOf(':') + 1).split(",", -1);
@@ -496,6 +561,10 @@ public class Options {
         setClassProperties(false);
       } else if (arg.equals("--swift-friendly")) {
         setSwiftFriendly(true);
+      } else if (arg.equals("--swift-enums")) {
+        setSwiftEnums(true);
+      } else if (arg.equals("--no-swift-enums")) {
+        setSwiftEnums(false);
       } else if (arg.equals("-processor")) {
         processors = getArgValue(args, arg);
       } else if (arg.equals("--allow-inherited-constructors")) {
@@ -504,8 +573,8 @@ public class Options {
         nullability = true;
       } else if (arg.equals("--no-nullability")) {
         nullability = false;
-      } else if (arg.equals("-Xdefault-nonnull")) {
-        defaultNonnull = true;
+      } else if (arg.equals("-Xnull-marked")) {
+        nullMarked = true;
       } else if (arg.startsWith("-Xlint")) {
         lintArgument = arg;
       } else if (arg.equals("-Xtranslate-bootclasspath")) {
@@ -520,10 +589,14 @@ public class Options {
         emitKytheMappings = true;
       } else if (arg.equals("-Xno-source-headers")) {
         emitSourceHeaders = false;
+      } else if (arg.equals("-Xprint-args")) {
+        printArgs = true;
       } else if (arg.equals("-external-annotation-file")) {
         addExternalAnnotationFile(getArgValue(args, arg));
       } else if (arg.equals("--reserved-names")) {
         NameTable.addReservedNames(getArgValue(args, arg));
+      } else if (arg.equals("--objc-generics")) {
+        asObjCGenericDecl = true;
       } else if (arg.equals("-version")) {
         version();
       } else if (arg.startsWith("-h") || arg.equals("--help")) {
@@ -532,7 +605,10 @@ public class Options {
         xhelp();
       } else if (arg.equals("-XDinjectLogSites=true")) {
         injectLogSites = true;
-      }  else if (arg.equals("-source")) {
+      } else if (arg.equals("-XDallVersions")) {
+        // For internal use only when adding new version support.
+        allVersions = true;
+      } else if (arg.equals("-source")) {
         String s = getArgValue(args, arg);
         // Handle aliasing of version numbers as supported by javac.
         try {
@@ -545,6 +621,8 @@ public class Options {
         getArgValue(args, arg);  // ignore
       } else if (PLATFORM_MODULE_SYSTEM_OPTIONS.contains(arg)) {
         addPlatformModuleSystemOptions(arg, getArgValue(args, arg));
+      } else if (arg.equals("--enable-preview")) {
+        addPlatformModuleSystemOptions(arg);
       } else if (arg.startsWith(BATCH_PROCESSING_MAX_FLAG)) {
         // Ignore, batch processing isn't used with javac front-end.
       } else if (obsoleteFlags.contains(arg)) {
@@ -629,11 +707,23 @@ public class Options {
     if (sourceVersion == null) {
       sourceVersion = SourceVersion.defaultVersion();
     }
-    SourceVersion maxVersion = SourceVersion.getMaxSupportedVersion();
-    if (sourceVersion.version() > maxVersion.version()) {
-      ErrorUtil.warning("Java " + sourceVersion.version() + " source version is not "
-          + "supported, using Java " + maxVersion.version() + ".");
-      sourceVersion = maxVersion;
+
+    if (allVersions) {
+      // Warn if using known but unsupported version.
+      if (sourceVersion.version() > SourceVersion.getMaxSupportedVersion().version()) {
+        ErrorUtil.warning("Using unsupported version: " + sourceVersion.version());
+      }
+    } else {
+      SourceVersion maxVersion = SourceVersion.getMaxSupportedVersion();
+      if (sourceVersion.version() > maxVersion.version()) {
+        ErrorUtil.warning(
+            "Java "
+                + sourceVersion.version()
+                + " is not installed, using Java "
+                + maxVersion.version()
+                + " as source version.");
+        sourceVersion = maxVersion;
+      }
     }
     if (sourceVersion.version() > 8) {
       // Allow the modularized JRE to read the J2ObjC annotations (they are in the unnamed module).
@@ -761,6 +851,54 @@ public class Options {
     return memoryManagementOption;
   }
 
+  public boolean useStrictFieldAssign() {
+    return strictFieldAssign;
+  }
+
+  @VisibleForTesting
+  public void setStrictFieldAssign(boolean b) {
+    strictFieldAssign = b;
+    if (!b && strictFieldLoad) {
+      strictFieldLoad = false;
+    }
+  }
+
+  public boolean useStrictFieldLoad() {
+    return strictFieldLoad;
+  }
+
+  @VisibleForTesting
+  public void setStrictFieldLoad(boolean b) {
+    if (b && !strictFieldAssign) {
+      strictFieldAssign = true;
+    }
+    strictFieldLoad = b;
+  }
+
+  public boolean useRetainAutoreleaseReturns() {
+    return retainAutoreleaseReturns;
+  }
+
+  @VisibleForTesting
+  public void setRetainAutoreleaseReturns(boolean b) {
+    retainAutoreleaseReturns = b;
+    if (!b) {
+      arcAutoreleaseReturns = false;
+    }
+  }
+
+  public boolean useARCAutoreleaseReturns() {
+    return arcAutoreleaseReturns;
+  }
+
+  @VisibleForTesting
+  public void setARCAutoreleaseReturns(boolean b) {
+    arcAutoreleaseReturns = b;
+    if (b) {
+      retainAutoreleaseReturns = true;
+    }
+  }
+
   @VisibleForTesting
   public void setMemoryManagementOption(MemoryManagementOption option) {
     memoryManagementOption = option;
@@ -1000,6 +1138,7 @@ public class Options {
   public void setSwiftFriendly(boolean b) {
     setClassProperties(b);
     setNullability(b);
+    setSwiftEnums(b);
   }
 
   public boolean nullability() {
@@ -1011,14 +1150,22 @@ public class Options {
     nullability = b;
   }
 
-  public boolean defaultNonnull() {
-    return nullability && defaultNonnull;
+  public boolean swiftEnums() {
+    return swiftEnums;
   }
 
   @VisibleForTesting
-  public void setDefaultNonnull(boolean b) {
-    nullability = true;
-    defaultNonnull = b;
+  public void setSwiftEnums(boolean b) {
+    swiftEnums = b;
+  }
+
+  public boolean nullMarked() {
+    return nullability && nullMarked;
+  }
+
+  @VisibleForTesting
+  public void setNullMarked(boolean b) {
+    nullMarked = b;
   }
 
   public String lintArgument() {
@@ -1098,4 +1245,39 @@ public class Options {
   public void setInjectLogSites(boolean b) {
     injectLogSites = b;
   }
+
+  public boolean asObjCGenericDecl() {
+    return asObjCGenericDecl;
+  }
+
+  @VisibleForTesting
+  public void setAsObjCGenericDecl(boolean b) {
+    asObjCGenericDecl = b;
+  }
+
+  public boolean ignoreJarWarnings() {
+    return ignoreJarWarnings;
+  }
+
+  public boolean linkSourcePathHeaders() {
+    return linkSourcePathHeaders;
+  }
+
+  @VisibleForTesting
+  public void setLinkSourcePathHeaders(boolean b) {
+    linkSourcePathHeaders = b;
+  }
+
+  public boolean javacWarnings() {
+    return javacWarnings;
+  }
+
+  @VisibleForTesting
+  public void setStripReflectionErrors(boolean b) {
+    stripReflectionErrors = b;
+  }
+
+  public boolean stripReflectionErrors() {
+    return stripReflectionErrors;
+  }
 }
