@@ -111,6 +111,7 @@ import com.google.devtools.j2objc.ast.VariableDeclarationStatement;
 import com.google.devtools.j2objc.ast.WhileStatement;
 import com.google.devtools.j2objc.translate.OcniExtractor;
 import com.google.devtools.j2objc.types.ExecutablePair;
+import com.google.devtools.j2objc.types.GeneratedExecutableElement;
 import com.google.devtools.j2objc.types.GeneratedPackageElement;
 import com.google.devtools.j2objc.types.GeneratedTypeElement;
 import com.google.devtools.j2objc.util.ElementUtil;
@@ -578,6 +579,8 @@ public class TreeConverter {
     // Flatten this tree to avoid stack overflow with very deep trees. This
     // code traverses the subtree non-recursively and merges all children
     // that have the same operator into this node.
+    //
+    // Note: we will not flatten the tree if the same operators are EQUAL_TO or NOT_EQUAL_TO.
     List<StackState> stack = Lists.newArrayList();
     stack.add(new StackState(node));
     while (!stack.isEmpty()) {
@@ -589,7 +592,9 @@ public class TreeConverter {
       }
       if (child instanceof BinaryTree) {
         BinaryTree infixChild = (BinaryTree) child;
-        if (infixChild.getKind() == node.getKind()) {
+        if (infixChild.getKind() == node.getKind()
+            && infixChild.getKind() != Kind.EQUAL_TO
+            && infixChild.getKind() != Kind.NOT_EQUAL_TO) {
           stack.add(new StackState(infixChild));
           continue;
         }
@@ -1096,10 +1101,10 @@ public class TreeConverter {
   private TreeNode convertMethodInvocation(MethodInvocationTree node, TreePath parent) {
     TreePath path = getTreePath(parent, node);
     ExpressionTree method = node.getMethodSelect();
-    TreePath methodPath = getTreePath(path, method);
+    TreePath methodInvocationPath = getTreePath(path, method);
     String methodName = getMemberName(method);
-    ExecutableType type = (ExecutableType) getTypeMirror(methodPath);
-    ExecutableElement element = (ExecutableElement) getElement(methodPath);
+    ExecutableType type = (ExecutableType) getTypeMirror(methodInvocationPath);
+    ExecutableElement element = (ExecutableElement) getElement(methodInvocationPath);
     ExpressionTree target =
         method.getKind() == Kind.MEMBER_SELECT ? ((MemberSelectTree) method).getExpression() : null;
 
@@ -1115,12 +1120,25 @@ public class TreeConverter {
     }
 
     if ("super".equals(methodName)) {
+      if (element == null && node.getArguments().isEmpty()) {
+        // If multiple classes have default constructors added, javac may not have an element
+        // defined for the super() invocation, so create an equivalent here.
+        TreePath methodPath = parent;
+        while (methodPath.getLeaf().getKind() != Kind.METHOD) {
+          methodPath = methodPath.getParentPath();
+        }
+        ExecutableElement enclosingMethod = (ExecutableElement) getElement(methodPath);
+        TypeMirror superclassType = ElementUtil.getDeclaringClass(enclosingMethod).getSuperclass();
+        TypeElement superclass = (TypeElement) ((DeclaredType) superclassType).asElement();
+        element =
+            GeneratedExecutableElement.newConstructor(superclass, newUnit.getEnv().typeUtil());
+      }
       SuperConstructorInvocation newNode =
           new SuperConstructorInvocation()
               .setExecutablePair(new ExecutablePair(element))
               .setVarargsType(((JCMethodInvocation) node).varargsElement);
       if (target != null) {
-        newNode.setExpression((Expression) convert(target, methodPath));
+        newNode.setExpression((Expression) convert(target, methodInvocationPath));
       }
       for (ExpressionTree arg : node.getArguments()) {
         newNode.addArgument((Expression) convert(arg, path));
@@ -1149,7 +1167,7 @@ public class TreeConverter {
 
     MethodInvocation newNode = new MethodInvocation();
     if (target != null) {
-      newNode.setExpression((Expression) convert(target, methodPath));
+      newNode.setExpression((Expression) convert(target, methodInvocationPath));
     }
     for (ExpressionTree arg : node.getArguments()) {
       newNode.addArgument((Expression) convert(arg, path));
