@@ -60,6 +60,10 @@ import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeMirror;
 
+// TODO: b/287612419 - Update methods to use nullability annotations. Methods are
+//                     not correctly annotated and throw an exceptions rather
+//                     than returning nil in many cases.
+
 /**
  * Modifies enum types for Objective C.
  *
@@ -67,6 +71,9 @@ import javax.lang.model.type.TypeMirror;
  */
 public class EnumRewriter extends UnitTreeVisitor {
 
+  /** Tag used to indicate that the enum name is stripped. */
+  private static final String ENUM_NAME_STRIPPED = "JAVA_LANG_ENUM_NAME_STRIPPED";
+
   public EnumRewriter(CompilationUnit unit) {
     super(unit);
   }
@@ -165,9 +172,10 @@ public class EnumRewriter extends UnitTreeVisitor {
     if (useNamesArray) {
       loopBody.addStatement(new NativeStatement(enumClassName
           + "_initWithNSString_withInt_(e, names[i], i);"));
-    } else if (options.stripEnumConstants()){
-      loopBody.addStatement(new NativeStatement(enumClassName
-          + "_initWithNSString_withInt_(e, JAVA_LANG_ENUM_NAME_STRIPPED, i);"));
+    } else if (options.stripEnumConstants()) {
+      String statementText =
+          enumClassName + "_initWithNSString_withInt_(e, " + ENUM_NAME_STRIPPED + ", i);";
+      loopBody.addStatement(new NativeStatement(statementText));
     } else {
       loopBody.addStatement(new NativeStatement(enumClassName
           + "_initWithNSString_withInt_(e, JreEnumConstantName(" + enumClassName
@@ -239,19 +247,15 @@ public class EnumRewriter extends UnitTreeVisitor {
   // ARC does not allow using "objc_constructInstance" so ARC code doesn't get
   // the shared allocation optimization.
   private void addArcInitialization(EnumDeclaration node) {
-    String enumClassName = nameTable.getFullName(node.getTypeElement());
     List<Statement> stmts = node.getClassInitStatements().subList(0, 0);
     int i = 0;
     for (EnumConstantDeclaration constant : node.getEnumConstants()) {
       VariableElement varElement = constant.getVariableElement();
       ClassInstanceCreation creation = new ClassInstanceCreation(constant.getExecutablePair());
       TreeUtil.copyList(constant.getArguments(), creation.getArguments());
-      Expression constName = options.stripEnumConstants()
-          ? new StringLiteral("JAVA_LANG_ENUM_NAME_STRIPPED", typeUtil)
-          : new NativeExpression(
-            UnicodeUtils.format("JreEnumConstantName(%s_class_(), %d)", enumClassName, i),
-            typeUtil.getJavaString().asType());
-      creation.addArgument(constName);
+      String stringLiteralName =
+          options.stripEnumConstants() ? ENUM_NAME_STRIPPED : ElementUtil.getName(varElement);
+      creation.addArgument(new StringLiteral(stringLiteralName, typeUtil));
       creation.addArgument(new NumberLiteral(i++, typeUtil));
       creation.setHasRetainedResult(true);
       stmts.add(new ExpressionStatement(new Assignment(new SimpleName(varElement), creation)));
@@ -324,36 +328,48 @@ public class EnumRewriter extends UnitTreeVisitor {
 
   private void addExtraNativeDecls(EnumDeclaration node) {
     String typeName = nameTable.getFullName(node.getTypeElement());
+    String nativeName = NameTable.getNativeEnumName(typeName);
+    String ordinalName = NameTable.getNativeOrdinalPreprocessorName(typeName);
     int numConstants = node.getEnumConstants().size();
 
     // The native type is not declared for an empty enum.
     if (numConstants > 0) {
-      String nativeName = NameTable.getNativeEnumName(typeName);
-      node.addBodyDeclaration(NativeDeclaration.newInnerDeclaration(
-          UnicodeUtils.format("- (%s)toNSEnum;\n", nativeName),
-          UnicodeUtils.format(
-              "- (%s)toNSEnum {\n"
-              + "  return (%s)[self ordinal];\n"
-              + "}\n\n", nativeName, nativeName)));
+      // Native toNSEnum always uses C type.
+      node.addBodyDeclaration(
+          NativeDeclaration.newInnerDeclaration(
+              UnicodeUtils.format("- (%s)toNSEnum;\n", nativeName),
+              UnicodeUtils.format(
+                  "- (%s)toNSEnum {\n" + "  return (%s)[self ordinal];\n" + "}\n\n",
+                  nativeName, nativeName)));
+
+      // Redeclare ordinal with the appropriate type.
+      node.addBodyDeclaration(
+          NativeDeclaration.newInnerDeclaration(
+              UnicodeUtils.format("- (%s)ordinal;\n", ordinalName),
+              UnicodeUtils.format(
+                  "- (%s)ordinal {\n" + "  return (%s)[super ordinal];\n" + "}\n\n",
+                  ordinalName, ordinalName)));
     }
 
     StringBuilder outerHeader = new StringBuilder();
     StringBuilder outerImpl = new StringBuilder();
 
-    outerHeader.append(UnicodeUtils.format(
-        "FOUNDATION_EXPORT %s *%s_fromOrdinal(NSUInteger ordinal);\n",
-        typeName, typeName));
+    String ordinalArgType = (numConstants > 0) ? ordinalName : "jint";
+    outerHeader.append(
+        UnicodeUtils.format(
+            "FOUNDATION_EXPORT %s *%s_fromOrdinal(%s ordinal);\n",
+            typeName, typeName, ordinalArgType));
+    outerImpl.append(
+        UnicodeUtils.format(
+            "%s *%s_fromOrdinal(%s ordinal) {\n", typeName, typeName, ordinalArgType));
 
-    outerImpl.append(UnicodeUtils.format(
-        "%s *%s_fromOrdinal(NSUInteger ordinal) {\n", typeName, typeName));
     // Avoid "comparison of unsigned expression >= 0 is always true" error.
     if (numConstants == 0) {
       outerImpl.append("  return nil;\n}\n");
     } else {
       outerImpl.append(UnicodeUtils.format(
           "  %s_initialize();\n"
-          // Param is unsigned, so don't need to check lower bound.
-          + "  if (ordinal >= %s) {\n"
+          + "  if (ordinal < 0 || ordinal >= %s) {\n"
           + "    return nil;\n"
           + "  }\n"
           + "  return %s_values_[ordinal];\n"
