@@ -88,14 +88,18 @@ public final class ElementUtil {
 
   private static final String LAZY_INIT = "com.google.errorprone.annotations.concurrent.LazyInit";
 
-  private static final Pattern NULLABLE_PATTERN = Pattern.compile("Nullable.*");
+  private static final Pattern NULLABLE_PATTERN =
+      Pattern.compile("Nullable.*|CheckForNull|ParametricNullness");
   private static final Pattern NONNULL_PATTERN = Pattern.compile("No[nt][Nn]ull.*");
 
   private final Elements javacElements;
   private final Map<Element, TypeMirror> elementTypeMap = new HashMap<>();
 
+  private final TypeElement javaObject;
+
   public ElementUtil(Elements javacElements) {
     this.javacElements = javacElements;
+    javaObject = javacElements.getTypeElement("java.lang.Object");
   }
 
   public static String getName(Element element) {
@@ -312,7 +316,7 @@ public final class ElementUtil {
   }
 
   /**
-   * Tests if this type element is private to it's source file. A public type declared
+   * Tests if this type element is private to its source file. A public type declared
    * within a private type is considered private.
    */
   public static boolean isPrivateInnerType(TypeElement type) {
@@ -578,6 +582,45 @@ public final class ElementUtil {
     return javacElements.getBinaryName(e).toString();
   }
 
+  public ExecutableElement getOriginalMethod(ExecutableElement method) {
+    TypeElement declaringClass = getDeclaringClass(method);
+    return getOriginalMethod(method, declaringClass, declaringClass);
+  }
+
+  /**
+   * Finds the original method element to use for generating a selector. The method returned is the
+   * first method found in the hierarchy while traversing in order of declared inheritance that
+   * doesn't override a method from a supertype. (ie. it is the first leaf node found in the tree of
+   * overriding methods)
+   */
+  public ExecutableElement getOriginalMethod(
+      ExecutableElement topMethod, TypeElement declaringClass, TypeElement currentType) {
+    if (currentType == null) {
+      return null;
+    }
+    TypeElement superclass =
+        currentType.getKind().isInterface() ? javaObject : getSuperclass(currentType);
+    ExecutableElement original = getOriginalMethod(topMethod, declaringClass, superclass);
+    if (original != null) {
+      return original;
+    }
+    for (TypeMirror supertype : currentType.getInterfaces()) {
+      original = getOriginalMethod(topMethod, declaringClass, TypeUtil.asTypeElement(supertype));
+      if (original != null) {
+        return original;
+      }
+    }
+    if (declaringClass == currentType) {
+      return topMethod;
+    }
+    for (ExecutableElement candidate : getMethods(currentType)) {
+      if (isInstanceMethod(candidate) && overrides(topMethod, candidate, declaringClass)) {
+        return candidate;
+      }
+    }
+    return null;
+  }
+
   Map<? extends ExecutableElement, ? extends AnnotationValue> getElementValuesWithDefaults(
       AnnotationMirror annotation) {
     DeclaredType type = annotation.getAnnotationType();
@@ -647,6 +690,19 @@ public final class ElementUtil {
     return isGeneratedAnnotation(mirror.getAnnotationType().asElement());
   }
 
+  public static boolean isToBeRemovedAnnotations(Element e, Options options) {
+    if (isAnnotationType(e)) {
+      if (options.stripReflection()) {
+        // If --strip-reflection flag is on, CLASS, SOURCE or RUNTIME annotations will be removed.
+        return true;
+      } else {
+        // If --strip-reflection flag is off, CLASS or SOURCE annotations will be removed.
+        return !isRuntimeAnnotation(e);
+      }
+    }
+    return false;
+  }
+
   public static boolean isGeneratedAnnotation(Element e) {
     // Use a negative check, since CLASS retention is the default.
     return isAnnotationType(e) && !hasRetentionPolicy(e, "SOURCE");
@@ -668,7 +724,7 @@ public final class ElementUtil {
   }
 
   public static AnnotationMirror getAnnotation(Element element, Class<?> annotationClass) {
-    return getQualifiedNamedAnnotation(element, annotationClass.getName());
+    return getQualifiedNamedAnnotation(element, annotationClass.getCanonicalName());
   }
 
   public static boolean hasAnnotation(Element element, Class<?> annotationClass) {
@@ -803,10 +859,39 @@ public final class ElementUtil {
     return options.getPackageInfoLookup().hasParametersAreNonnullByDefault(pkgName);
   }
 
+  public boolean isNullMarked(Element element, Options options) {
+    if (!options.nullMarked()) {
+      return false;
+    }
+    if (ElementUtil.isAnnotatedWithNullMarked(element)) {
+      return true;
+    }
+    PackageElement pkg = getPackage(element);
+    if (ElementUtil.isAnnotatedWithNullMarked(pkg)) {
+      return true;
+    }
+    String pkgName = pkg.getQualifiedName().toString();
+    return options.getPackageInfoLookup().isNullMarked(pkgName);
+  }
+
+  // TODO: b/286259161 - Remove the below method and check for the NullMarked class once
+  //                     the org.jspecify.annotations migration is complete.
   /**
-   * Returns true if there's a SuppressedWarning annotation with the specified warning.
-   * The SuppressWarnings annotation can be inherited from the owning method or class,
-   * but does not have package scope.
+   * Returns {@code true} when {@code element} is annotated with {@code @NullMarked}.
+   *
+   * <p><b>Note:</b> Qualified names are checked rather than classes to ensure that annotations are
+   * correctly promoted to generated code for clients which reference the legacy
+   * `org.jspecify.nullness.NullMarked` package.
+   */
+  private static boolean isAnnotatedWithNullMarked(Element element) {
+    return hasQualifiedNamedAnnotation(element, "org.jspecify.nullness.NullMarked")
+        || hasQualifiedNamedAnnotation(element, "org.jspecify.annotations.NullMarked");
+  }
+
+  /**
+   * Returns true if there's a SuppressedWarning annotation with the specified warning. The
+   * SuppressWarnings annotation can be inherited from the owning method or class, but does not have
+   * package scope.
    */
   @SuppressWarnings("unchecked")
   public static boolean suppressesWarning(String warning, Element element) {
