@@ -16,6 +16,8 @@
 
 package com.google.devtools.j2objc.util;
 
+
+import com.google.common.base.Ascii;
 import com.google.common.base.CharMatcher;
 import com.google.common.base.Joiner;
 import com.google.common.base.Splitter;
@@ -42,12 +44,17 @@ import java.util.function.Supplier;
 import java.util.logging.Logger;
 import java.util.regex.Pattern;
 import javax.lang.model.element.AnnotationMirror;
+import javax.lang.model.element.Element;
+import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
+import javax.lang.model.element.TypeParameterElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.ArrayType;
+import javax.lang.model.type.DeclaredType;
 import javax.lang.model.type.TypeMirror;
+import javax.lang.model.type.TypeVariable;
 
 /**
  * Singleton service for type/method/variable name support.
@@ -202,6 +209,10 @@ public class NameTable {
       return name;
     }
     name = ElementUtil.getName(var);
+    if (name.equals("initialize")) {
+      // Avoid conflict with class initialize function name.
+      return "initialize_";
+    }
     if (allowReservedName) {
       return name;
     }
@@ -269,6 +280,11 @@ public class NameTable {
     return s.length() > 0 ? Character.toUpperCase(s.charAt(0)) + s.substring(1) : s;
   }
 
+  /** Lowercase the first letter of a string. */
+  public static String lowercaseFirst(String s) {
+    return s.length() > 0 ? Character.toLowerCase(s.charAt(0)) + s.substring(1) : s;
+  }
+
   /**
    * Given a period-separated name, return as a camel-cased type name.  For
    * example, java.util.logging.Level is returned as JavaUtilLoggingLevel.
@@ -292,6 +308,26 @@ public class NameTable {
     return sb.toString();
   }
 
+  /**
+   * Given a name, return as a camel-cased name using underscores as spaces. Used, for example, in
+   * swift name for enums.
+   */
+  public static String getSwiftEnumName(String fqn) {
+    StringBuilder sb = new StringBuilder();
+    for (String part : Splitter.on('_').split(fqn)) {
+      String caseName = Ascii.toLowerCase(part);
+      if (sb.length() == 0) {
+        sb.append(caseName);
+      } else {
+        sb.append(capitalize(caseName));
+      }
+    }
+    if (isReservedName(sb.toString())) {
+      sb.append("_");
+    }
+    return sb.toString();
+  }
+
   private static final Pattern FAMILY_METHOD_REGEX =
       Pattern.compile("^[_]*(new|copy|alloc|init|mutableCopy).*");
 
@@ -405,7 +441,7 @@ public class NameTable {
       return selector;
     }
     if (ElementUtil.isInstanceMethod(method)) {
-      method = getOriginalMethod(method);
+      method = elementUtil.getOriginalMethod(method);
     }
     selector = getRenamedMethodName(method);
     return selectorForMethodName(method, selector != null ? selector : getMethodName(method));
@@ -484,46 +520,6 @@ public class NameTable {
     return null;
   }
 
-  private ExecutableElement getOriginalMethod(ExecutableElement method) {
-    TypeElement declaringClass = ElementUtil.getDeclaringClass(method);
-    return getOriginalMethod(method, declaringClass, declaringClass);
-  }
-
-  /**
-   * Finds the original method element to use for generating a selector. The method returned is the
-   * first method found in the hierarchy while traversing in order of declared inheritance that
-   * doesn't override a method from a supertype. (ie. it is the first leaf node found in the tree of
-   * overriding methods)
-   */
-  private ExecutableElement getOriginalMethod(
-      ExecutableElement topMethod, TypeElement declaringClass, TypeElement currentType) {
-    if (currentType == null) {
-      return null;
-    }
-    TypeElement superclass = currentType.getKind().isInterface()
-        ? typeUtil.getJavaObject() : ElementUtil.getSuperclass(currentType);
-    ExecutableElement original = getOriginalMethod(topMethod, declaringClass, superclass);
-    if (original != null) {
-      return original;
-    }
-    for (TypeMirror supertype : currentType.getInterfaces()) {
-      original = getOriginalMethod(topMethod, declaringClass, TypeUtil.asTypeElement(supertype));
-      if (original != null) {
-        return original;
-      }
-    }
-    if (declaringClass == currentType) {
-      return topMethod;
-    }
-    for (ExecutableElement candidate : ElementUtil.getMethods(currentType)) {
-      if (ElementUtil.isInstanceMethod(candidate)
-          && elementUtil.overrides(topMethod, candidate, declaringClass)) {
-        return candidate;
-      }
-    }
-    return null;
-  }
-
   /**
    * Converts a Java type to an equivalent Objective-C type, returning "id" for an object type.
    */
@@ -533,20 +529,45 @@ public class NameTable {
   }
 
   /**
-   * Convert a Java type to an equivalent Objective-C type with type variables
-   * resolved to their bounds.
+   * Converts a primitive Java type to an equivalent Objective-C type as the most inner component in
+   * an array.
    */
-  public String getObjCType(TypeMirror type) {
-    return getObjcTypeInner(type, null);
+  public static String getPrimitiveObjCTypeArrayComponent(TypeMirror type) {
+    String typeString = "JavaLang";
+    switch (type.getKind()) {
+      case BOOLEAN:
+        typeString += "Boolean";
+        break;
+      case BYTE:
+        typeString += "Byte";
+        break;
+      case SHORT:
+        typeString += "Short";
+        break;
+      case INT:
+        typeString += "Integer";
+        break;
+      case LONG:
+        typeString += "Long";
+        break;
+      case CHAR:
+        typeString += "Character";
+        break;
+      case FLOAT:
+        typeString += "Float";
+        break;
+      case DOUBLE:
+        typeString += "Double";
+        break;
+      default:
+        return getPrimitiveObjCType(type);
+    }
+
+    typeString += " *";
+    return typeString;
   }
 
-  public String getObjCType(VariableElement var) {
-    return getObjcTypeInner(var.asType(), ElementUtil.getTypeQualifiers(var));
-  }
-
-  /**
-   * Convert a Java type into the equivalent JNI type.
-   */
+  /** Convert a Java type into the equivalent JNI type. */
   public String getJniType(TypeMirror type) {
     if (TypeUtil.isPrimitiveOrVoid(type)) {
       return getPrimitiveObjCType(type);
@@ -560,8 +581,90 @@ public class NameTable {
     return "jobject";
   }
 
-  private String getObjcTypeInner(TypeMirror type, String qualifiers) {
+  /**
+   * Convert a Java type to an equivalent Objective-C type with type variables resolved to their
+   * bounds.
+   */
+  public String getObjCType(TypeMirror type) {
+    return getObjcTypeInner(type, null, false, false, null);
+  }
+
+  public String getObjCType(VariableElement var) {
+    return getObjcTypeInner(
+        var.asType(), ElementUtil.getTypeQualifiers(var), false, false, null);
+  }
+
+  public String getObjCTypeDeclaration(TypeMirror type) {
+    return getObjcTypeInner(type, null, false, false, null);
+  }
+
+  public String getObjCTypeDeclaration(
+      TypeMirror type,
+      boolean enableGenerics,
+      TypeElement genericUsageTypeElement) {
+    return getObjcTypeInner(
+        type, null, false, enableGenerics, genericUsageTypeElement);
+  }
+
+  /**
+   * Given a Java class type, returns the generics that would apply to the class declaration as a
+   * list of string names.
+   */
+  public List<String> getClassObjCGenericTypeNames(TypeMirror type) {
+    TypeElement typeElement = TypeUtil.asTypeElement(type);
+    if (typeElement == null || !TypeUtil.isClass(type)) {
+      return new ArrayList<>();
+    }
+    // Java protos have the <MessageType, BuilderType> generics, but these are not useful
+    // in ObjC interfacing with the Java, so filter them off.
+    if (typeUtil.isProtoClass(type)) {
+      return new ArrayList<>();
+    }
+    ArrayList<String> typeNames = new ArrayList<>();
+    for (TypeParameterElement paramElement : typeElement.getTypeParameters()) {
+      typeNames.add(paramElement.getSimpleName().toString());
+    }
+    return typeNames;
+  }
+
+  private boolean isTranslatableTypeVariable(
+      TypeVariable t, TypeElement parameterUsageContextTypeElement) {
+    TypeParameterElement typeParamElement = TypeUtil.asTypeParameterElement(t);
+    if (typeParamElement == null) {
+      return false;
+    }
+
+    Element enclosingElement = typeParamElement.getEnclosingElement();
+    // Skip method type parameters.
+    if (enclosingElement.getKind() != ElementKind.CLASS) {
+      return false;
+    }
+
+    // If there is a usage context class is the parameter type also there?
+    // This avoids cases we don't yet handle, notably inner classes using the parameters
+    // of their parent class.
+    if (parameterUsageContextTypeElement != null) {
+      if (!TypeUtil.isClass(parameterUsageContextTypeElement.asType())) {
+        return false;
+      }
+      if (!parameterUsageContextTypeElement.getTypeParameters().contains(typeParamElement)) {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  // isArrayComponent: this flag is used to generate the primitive types if they are the most inner
+  /// component in arrays.
+  private String getObjcTypeInner(
+      TypeMirror type,
+      String qualifiers,
+      boolean isArrayComponent,
+      boolean enableGenerics,
+      TypeElement genericUsageTypeElement) {
     String objcType;
+    type = TypeUtil.unannotatedType(type);
     if (type instanceof NativeType) {
       objcType = ((NativeType) type).getName();
     } else if (type instanceof PointerType) {
@@ -573,10 +676,78 @@ public class NameTable {
           qualifiers = qualifiers.substring(idx + 1);
         }
       }
-      objcType = getObjcTypeInner(((PointerType) type).getPointeeType(), pointeeQualifiers);
+      objcType =
+          getObjcTypeInner(
+              ((PointerType) type).getPointeeType(),
+              pointeeQualifiers,
+              false,
+              enableGenerics,
+              genericUsageTypeElement);
       objcType = objcType.endsWith("*") ? objcType + "*" : objcType + " *";
     } else if (TypeUtil.isPrimitiveOrVoid(type)) {
-      objcType = getPrimitiveObjCType(type);
+      objcType =
+          isArrayComponent ? getPrimitiveObjCTypeArrayComponent(type) : getPrimitiveObjCType(type);
+    } else if (type instanceof ArrayType && enableGenerics) {
+      TypeMirror componentType = TypeUtil.unannotatedType(((ArrayType) type).getComponentType());
+      // For arrays of primitive types we prefer the IOSPrimitiveArray.h IOSArray subclasses
+      // directly instead of IOSArray with generics. The IOSArray type-specific subclasses offer
+      // more useful methods for accessing the primitive types the array contains with appropriate
+      // Obj-C method signatures and type encodings. For example, -[IOSBooleanArray booleanAtIndex:]
+      // can't be implemented as  -[IOSArray getElementAtIndex:] due to ObjC selector signature
+      // requirements.
+      if (TypeUtil.isPrimitiveOrVoid(componentType)) {
+        TypeElement arrayTypeElement = typeUtil.getIosArray(componentType);
+        objcType = getFullName(arrayTypeElement);
+        objcType += isArrayComponent ? "" : " *";
+      } else {
+        String innerType =
+            getObjcTypeInner(
+                componentType,
+                qualifiers,
+                true,
+                enableGenerics,
+                genericUsageTypeElement);
+        objcType = "IOSObjectArray<" + innerType;
+        objcType += componentType instanceof ArrayType ? " *>" : ">";
+        objcType += isArrayComponent ? "" : " *";
+      }
+    } else if (enableGenerics
+        && ((genericUsageTypeElement != null) && !genericUsageTypeElement.getKind().isInterface())
+        && TypeUtil.isTypeVariable(type)
+        && isTranslatableTypeVariable((TypeVariable) type, genericUsageTypeElement)) {
+      objcType = ((TypeVariable) type).asElement().getSimpleName().toString();
+    } else if (TypeUtil.isDeclaredType(type)
+        && enableGenerics
+        && !((DeclaredType) type).getTypeArguments().isEmpty()
+        && !TypeUtil.isInterface(type)
+        && !typeUtil.isProtoClass(type)
+        && !typeUtil.isClassType(TypeUtil.asTypeElement(type))) {
+      final String finalQualifiers = qualifiers;
+      // Avoid creating generics that merely state <id,id,...>. Obj-C generics only support 'id'
+      // type anyway, but in practice a type with just "id" parameters cannot be assigned to a more
+      // specific type where Java would sometimes allow such conversions.
+      List<String> argTypes = new ArrayList<>();
+      boolean hasSpecficType = false;
+      for (TypeMirror argTypeMirror : ((DeclaredType) type).getTypeArguments()) {
+        String argType =
+            getObjcTypeInner(
+                argTypeMirror,
+                finalQualifiers,
+                false,
+                enableGenerics,
+                genericUsageTypeElement);
+        argTypes.add(argType);
+        if (!argType.equals(ID_TYPE)) {
+          hasSpecficType = true;
+        }
+      }
+      if (hasSpecficType) {
+        objcType =
+            String.format(
+                "%s<%s> *", getFullName(typeUtil.getObjcClass(type)), String.join(", ", argTypes));
+      } else {
+        objcType = String.format("%s *", getFullName(typeUtil.getObjcClass(type)));
+      }
     } else {
       objcType = constructObjcTypeFromBounds(type);
     }
@@ -608,6 +779,10 @@ public class NameTable {
     return typeName + "_Enum";
   }
 
+  public static String getNativeOrdinalPreprocessorName(String typeName) {
+    return typeName + "_ORDINAL";
+  }
+
   /**
    * Return the full name of a type, including its package.  For outer types,
    * is the type's full name; for example, java.lang.Object's full name is
@@ -625,6 +800,10 @@ public class NameTable {
     return fullName;
   }
 
+  public static boolean isReservedName(String name) {
+    return reservedNames.contains(name) || nsObjectMessages.contains(name);
+  }
+
   private String getFullNameImpl(TypeElement element) {
     // Avoid package prefix renaming for package-info types, and use a valid ObjC name that doesn't
     // have a dash character.
@@ -669,9 +848,6 @@ public class NameTable {
     return ElementUtil.getName(element).replace('$', '_');
   }
 
-  private static boolean isReservedName(String name) {
-    return reservedNames.contains(name) || nsObjectMessages.contains(name);
-  }
 
   private String getPrefix(PackageElement packageElement) {
     return prefixMap.getPrefix(packageElement);
