@@ -16,12 +16,15 @@ package com.google.devtools.j2objc.util;
 
 import com.google.common.base.Joiner;
 import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.ImmutableSet;
 import com.google.devtools.j2objc.types.AbstractTypeMirror;
 import com.google.devtools.j2objc.types.ExecutablePair;
 import com.google.devtools.j2objc.types.GeneratedArrayType;
 import com.google.devtools.j2objc.types.GeneratedTypeElement;
 import com.google.devtools.j2objc.types.NativeType;
 import com.google.devtools.j2objc.types.PointerType;
+import com.google.j2objc.annotations.GenerateObjectiveCGenerics;
+import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.EnumMap;
@@ -137,7 +140,7 @@ public final class TypeUtil {
       typeMapBuilder.put(typeNSFastEnumeration, NS_FASTENUMERATION);
     }
 
-    javaToObjcTypeMap = typeMapBuilder.build();
+    javaToObjcTypeMap = typeMapBuilder.buildOrThrow();
   }
 
   public ElementUtil elementUtil() {
@@ -208,6 +211,14 @@ public final class TypeUtil {
     return t.getKind() == TypeKind.TYPEVAR;
   }
 
+  public static boolean hasGenerateObjectiveCGenerics(TypeElement t) {
+    return ElementUtil.hasAnnotation(t, GenerateObjectiveCGenerics.class);
+  }
+
+  public static boolean hasGenerateObjectiveCGenerics(TypeMirror t) {
+    return asTypeElement(t) != null && hasGenerateObjectiveCGenerics(asTypeElement(t));
+  }
+
   public static TypeElement asTypeElement(TypeMirror t) {
     if (isDeclaredType(t)) {
       return (TypeElement) ((DeclaredType) t).asElement();
@@ -445,8 +456,10 @@ public final class TypeUtil {
       return getIosArray(((ArrayType) t).getComponentType());
     } else if (isDeclaredType(t)) {
       return getObjcClass((TypeElement) ((DeclaredType) t).asElement());
+    } else if (t.getKind().isPrimitive()) {
+      return boxedClass((PrimitiveType) t);
     } else if (t.getKind() == TypeKind.UNION) {
-      TypeMirror lub = leastUpperBound(((UnionType)t).getAlternatives());
+      TypeMirror lub = leastUpperBound(((UnionType) t).getAlternatives());
       return getObjcClass(asTypeElement(lub));
     }
     return null;
@@ -696,6 +709,29 @@ public final class TypeUtil {
     return true;
   }
 
+  public boolean isProtoClass(TypeMirror type) {
+    ImmutableSet<String> protoClassNames =
+        ImmutableSet.of(
+            "com.google.protobuf.GeneratedMessage",
+            "com.google.protobuf.GeneratedMessageLite",
+            "com.google.protobuf.MessageOrBuilder",
+            "com.google.protobuf.MessageLiteOrBuilder",
+            "com.google.protobuf.Extension",
+            "com.google.protobuf.ExtensionLite");
+
+    TypeElement element = asTypeElement(type);
+    if (element != null && protoClassNames.contains(element.getQualifiedName().toString())) {
+      return true;
+    }
+    for (TypeMirror t : directSupertypes(type)) {
+      boolean result = isProtoClass(t);
+      if (result) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   public boolean isObjcSubtype(TypeElement type, TypeElement targetSupertype) {
     if (type == null) {
       return false;
@@ -719,7 +755,7 @@ public final class TypeUtil {
     List<TypeElement> result = new ArrayList<>();
     for (TypeMirror bound : getUpperBounds(t)) {
       TypeElement elem = getObjcClass(bound);
-      // NSObject is emmitted as "id".
+      // NSObject is emitted as "id".
       if (elem != null && !elem.equals(NS_OBJECT)) {
         result.add(elem);
       }
@@ -744,6 +780,22 @@ public final class TypeUtil {
     }
   }
 
+  public static boolean hasBounds(TypeMirror t) {
+    if (t == null) {
+      return false;
+    }
+    switch (t.getKind()) {
+      case TYPEVAR:
+        return ((TypeVariable) t).getUpperBound() != null
+            || ((TypeVariable) t).getLowerBound() != null;
+      case WILDCARD:
+        return ((WildcardType) t).getExtendsBound() != null
+            || ((WildcardType) t).getSuperBound() != null;
+      default:
+        return false;
+    }
+  }
+
   public static String getName(TypeMirror t) {
     switch (t.getKind()) {
       case ARRAY:
@@ -938,4 +990,28 @@ public final class TypeUtil {
     // Currently only NSException and NSFastEnumeration have com.google.j2objc stubs.
     return typeName.startsWith("com.google.j2objc.NS");
   }
+
+  // Remove javac 1.8 AnnotatedType wrapper, if present.
+  public static TypeMirror unannotatedType(TypeMirror type) {
+    if (annotatedTypeClass != null) {
+      try {
+        if (annotatedTypeClass.isInstance(type)) {
+          Method m = annotatedTypeClass.getDeclaredMethod("unannotatedType");
+          return (TypeMirror) m.invoke(type);
+        }
+      } catch (Exception e) {
+        // Fall-through, as any error shows no unannotated type is available.
+      }
+    }
+    return type;
+  }
+
+  private static Class<?> annotatedTypeClass;
+  static {
+    try {
+      annotatedTypeClass = Class.forName("com.sun.tools.javac.code.Type$AnnotatedType");
+    } catch (ClassNotFoundException e) {
+      annotatedTypeClass = null;
+    }
+  }
 }
