@@ -16,6 +16,7 @@
 
 package com.google.devtools.j2objc;
 
+import static com.google.common.truth.Truth.assertThat;
 import static java.nio.charset.StandardCharsets.UTF_8;
 
 import com.google.common.base.Joiner;
@@ -74,13 +75,14 @@ import javax.tools.ToolProvider;
 import junit.framework.TestCase;
 
 /**
- * Tests code generation. A string containing the source code for a list of Java
- * statements is parsed and translated, then iOS code is generated for one or
- * more of those statements for comparison in a specific generation test.
+ * Test support and assertions for j2objc code generation. The general design involves transpiling a
+ * string containing source code from one or more Java source file snippets, then verifying that the
+ * iOS code is generated as expected. Options (flags) may be set before transpilation. When a test
+ * fails, the full generated .m or .h file is included in the failure message.
  *
  * @author Tom Ball
  */
-public class GenerationTest extends TestCase {
+public abstract class GenerationTest extends TestCase {
 
   protected File tempDir;
   protected Parser parser;
@@ -514,20 +516,21 @@ public class GenerationTest extends TestCase {
     String source = "public class Test { " + method + " }";
     CompilationUnit unit = translateType("Test", source);
     final MethodDeclaration[] result = new MethodDeclaration[1];
-    unit.accept(new TreeVisitor() {
-      @Override
-      public boolean visit(MethodDeclaration node) {
-        String name = ElementUtil.getName(node.getExecutableElement());
-        if (name.equals(NameTable.INIT_NAME)
-            || name.equals(NameTable.FINALIZE_METHOD)
-            || name.equals(NameTable.DEALLOC_METHOD)) {
-          return false;
-        }
-        assert result[0] == null;
-        result[0] = node;
-        return false;
-      }
-    });
+    unit.accept(
+        new TreeVisitor() {
+          @Override
+          public boolean visit(MethodDeclaration node) {
+            String name = ElementUtil.getName(node.getExecutableElement());
+            if (name.equals(NameTable.INIT_NAME)
+                || name.equals(NameTable.FINALIZE_METHOD)
+                || name.equals(NameTable.DEALLOC_METHOD)) {
+              return false;
+            }
+            assertThat(result[0]).isNull();
+            result[0] = node;
+            return false;
+          }
+        });
     return result[0];
   }
 
@@ -624,8 +627,8 @@ public class GenerationTest extends TestCase {
 
   protected void runPipeline(String... files) {
     J2ObjC.run(Arrays.asList(files), options);
-    assertErrorCount(0);
-    assertWarningCount(0);
+    assertNoErrors();
+    assertNoWarnings();
   }
 
   protected void loadHeaderMappings() {
@@ -690,28 +693,71 @@ public class GenerationTest extends TestCase {
     return file.getPath();
   }
 
-  /**
-   * Asserts that the correct number of warnings were reported during the
-   * last translation.
-   */
-  protected void assertWarningCount(int expectedCount) {
-    if (expectedCount != ErrorUtil.warningCount()) {
+  /** Asserts that a specific warning was reported during the last translation. */
+  protected void assertWarning(String warning) {
+    if (!ErrorUtil.getWarningMessages().contains(warning)) {
       failWithMessages(
-          String.format("Wrong number of warnings. Expected:%d but was:%d",
-                        expectedCount, ErrorUtil.warningCount()),
-          ErrorUtil.getWarningMessages());
+          "Expected warning was not reported: \"" + warning + "\"", ErrorUtil.getWarningMessages());
     }
   }
 
   /**
-   * Asserts that the correct number of errors were reported during the
-   * last translation.
+   * Asserts that a specific warning was reported during the last translation. This assertion is
+   * used when a warning has build-specific information, such as a failing file name.
    */
-  protected void assertErrorCount(int expectedCount) {
-    if (expectedCount != ErrorUtil.errorCount()) {
+  protected void assertWarningRegex(String regex) {
+    for (String message : ErrorUtil.getWarningMessages()) {
+      if (message.matches(regex)) {
+        return; // Warning found.
+      }
       failWithMessages(
-          String.format("Wrong number of errors. Expected:%d but was:%d",
-                        expectedCount, ErrorUtil.errorCount()),
+          "Expected warning was not reported: \"" + regex + "\"", ErrorUtil.getWarningMessages());
+    }
+  }
+
+  /** Asserts that no warnings were reported during the last translation. */
+  protected void assertNoWarnings() {
+    // TODO(tball): remove when b/261217081 is fixed.
+    int warningCount = ErrorUtil.warningCount();
+    for (String message : ErrorUtil.getWarningMessages()) {
+      if (message.startsWith("system modules path not set")) {
+        --warningCount;
+      }
+    }
+    if (warningCount > 0) {
+      failWithMessages(
+          String.format("No warnings were expected, but there were %d", warningCount),
+          ErrorUtil.getWarningMessages());
+    }
+  }
+
+  /** Asserts that a specific error was reported during the last translation. */
+  protected void assertError(String error) {
+    if (!ErrorUtil.getErrorMessages().contains(error)) {
+      failWithMessages(
+          "Expected error was not reported: \"" + error + "\"", ErrorUtil.getErrorMessages());
+    }
+  }
+
+  /**
+   * Asserts that a specific error was reported during the last translation. This assertion is used
+   * when an error has build-specific information, such as a failing file name.
+   */
+  protected void assertErrorRegex(String regex) {
+    for (String message : ErrorUtil.getErrorMessages()) {
+      if (message.matches(regex)) {
+        return; // Error found.
+      }
+      failWithMessages(
+          "Expected error was not reported: \"" + regex + "\"", ErrorUtil.getErrorMessages());
+    }
+  }
+
+  /** Asserts that no errors were reported during the last translation. */
+  protected void assertNoErrors() {
+    if (ErrorUtil.errorCount() > 0) {
+      failWithMessages(
+          String.format("No errors were expected, but there were %d", ErrorUtil.errorCount()),
           ErrorUtil.getErrorMessages());
     }
   }
@@ -803,7 +849,4 @@ public class GenerationTest extends TestCase {
             .trim();
     return new String(Base64.getDecoder().decode(encodedMetadata), UTF_8);
   }
-
-  // Empty test so Bazel won't report a "no tests" error.
-  public void testNothing() {}
 }
