@@ -16,6 +16,8 @@
 
 package com.google.devtools.j2objc.gen;
 
+import static com.google.common.truth.Truth.assertThat;
+
 import com.google.devtools.j2objc.GenerationTest;
 import com.google.devtools.j2objc.util.HeaderMap;
 import java.io.File;
@@ -36,7 +38,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         + "class MyClass {}", "Example", "mypackage/Example.h");
     assertTranslation(translation, "@interface MypackageExample");
     // enum declaration
-    assertTranslation(translation, "typedef NS_ENUM(NSUInteger, MypackageAbcd_Enum) {");
+    assertTranslation(translation, "typedef NS_ENUM(jint, MypackageAbcd_Enum) {");
     assertTranslation(translation, "@interface MypackageAbcd");
     assertTranslation(translation, "@interface MypackageMyClass");
     assertTranslation(translation, "MypackageMyClass *myclass_;");
@@ -270,7 +272,8 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     String header = translateCombinedFiles(
         "unit/Foo", ".h",
         "unit/TestDependent.java", "unit/AnotherTest.java", "unit/Test.java");
-    assert header.indexOf("@interface UnitTest") < header.indexOf("@interface UnitAnotherTest");
+    assertThat(header.indexOf("@interface UnitTest")
+        < header.indexOf("@interface UnitAnotherTest")).isTrue();
   }
 
   public void testCombinedJarHeaderMapping() throws IOException {
@@ -485,8 +488,32 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     String translation = translateSourceFile(
       "public enum Color { RED, WHITE, BLUE }",
       "Color", "Color.h");
-    assertTranslatedLines(translation,
-        "typedef NS_ENUM(NSUInteger, Color_Enum) {",
+    assertTranslatedLines(
+        translation,
+        "typedef NS_ENUM(jint, Color_Enum) {",
+        "  Color_Enum_RED NS_SWIFT_NAME(red) = 0,",
+        "  Color_Enum_WHITE NS_SWIFT_NAME(white) = 1,",
+        "  Color_Enum_BLUE NS_SWIFT_NAME(blue) = 2,",
+        "};");
+    assertTranslation(translation, "@interface Color : JavaLangEnum");
+    assertTranslation(translation, "+ (IOSObjectArray *)values;");
+    assertTranslation(translation, "+ (Color *)valueOfWithNSString:(NSString *)name;");
+    assertTranslation(translation, "FOUNDATION_EXPORT Color *Color_values_[];");
+    assertTranslatedLines(
+        translation, "inline Color *Color_get_RED(void);", "J2OBJC_ENUM_CONSTANT(Color, RED)");
+    assertTranslatedLines(
+        translation, "inline Color *Color_get_WHITE(void);", "J2OBJC_ENUM_CONSTANT(Color, WHITE)");
+    assertTranslatedLines(
+        translation, "inline Color *Color_get_BLUE(void);", "J2OBJC_ENUM_CONSTANT(Color, BLUE)");
+  }
+
+  public void testEnumNoSwiftEnums() throws IOException {
+    options.setSwiftEnums(false);
+    String translation =
+        translateSourceFile("public enum Color { RED, WHITE, BLUE }", "Color", "Color.h");
+    assertTranslatedLines(
+        translation,
+        "typedef NS_ENUM(jint, Color_Enum) {",
         "  Color_Enum_RED = 0,",
         "  Color_Enum_WHITE = 1,",
         "  Color_Enum_BLUE = 2,",
@@ -506,6 +533,25 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         "J2OBJC_ENUM_CONSTANT(Color, BLUE)");
   }
 
+  public void testEnumIsImplicitlyNonnullWhenNullMarked() throws IOException {
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation =
+        translateSourceFile("public enum Color { RED, WHITE, BLUE }", "Color", "Color.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface Color : JavaLangEnum");
+    assertTranslatedLines(
+        translation,
+        "typedef NS_ENUM(jint, Color_Enum) {",
+        "  Color_Enum_RED NS_SWIFT_NAME(red) = 0,",
+        "  Color_Enum_WHITE NS_SWIFT_NAME(white) = 1,",
+        "  Color_Enum_BLUE NS_SWIFT_NAME(blue) = 2,",
+        "};");
+    assertTranslation(translation, "FOUNDATION_EXPORT Color *_Nonnull Color_values_[];");
+    assertTranslatedLines(
+        translation, "J2OBJC_TYPE_LITERAL_HEADER(Color)", "", "NS_ASSUME_NONNULL_END");
+  }
+
   public void testEnumWithParameters() throws IOException {
     String translation = translateSourceFile(
         "public enum Color { RED(0xff0000), WHITE(0xffffff), BLUE(0x0000ff); "
@@ -608,7 +654,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     String translation = translateSourceFile(
         "public enum MyEnum { ONE, TWO, THREE }",
         "MyEnum", "MyEnum.h");
-    assertTranslation(translation, "typedef NS_ENUM(NSUInteger, MyEnum_Enum) {");
+    assertTranslation(translation, "typedef NS_ENUM(jint, MyEnum_Enum) {");
     assertTranslation(translation, "@interface MyEnum : JavaLangEnum");
     assertTranslation(translation, "FOUNDATION_EXPORT MyEnum *MyEnum_values_[];");
     assertTranslation(translation, "inline MyEnum *MyEnum_get_ONE(void);");
@@ -693,6 +739,162 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         "FooBar_Internal *fieldFoo_;");
   }
 
+  public void testPropertiesOfGetTypes() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "public class FooBar {"
+            + "  private String fieldFoo = \"test\";"
+            + "  "
+            + "  @Property"
+            + "  public String getFooField() {"
+            + "     return fieldFoo;"
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertTranslatedLines(
+        translation, "@property (nonatomic, getter=getFooField, readonly) NSString * fooField;");
+  }
+
+  public void testPropertiesOfGetTypesWithSetters() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "public class FooBar {"
+            + "  private String fieldFoo = \"test\";"
+            + "  "
+            + "  @Property"
+            + "  public String getFooField() {"
+            + "     return fieldFoo;"
+            + "  }"
+            + "  "
+            + "  public void setFooField(String fooField) {"
+            + "     this.fieldFoo = fooField;"
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertTranslatedLines(
+        translation,
+        "@property (nonatomic, getter=getFooField, setter=setFooFieldWithNSString:) NSString *"
+            + " fooField;");
+  }
+
+  public void testPropertyAnnotationSuppression() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "@Property "
+            + "public class FooBar {"
+            + "  "
+            + "  public String getFieldFoo() {"
+            + "     return \"\";"
+            + "  }"
+            + "  "
+            + "  public void setFieldFoo(String fooField) {"
+            + ""
+            + "  }"
+            + " "
+            + "  @Property.Suppress "
+            + "  public String getBar() {"
+            + "     return \"\";"
+            + "  }"
+            + "  "
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertNotInTranslation(
+        translation, "@property (nonatomic, getter=getBar, readonly) NSString * bar;");
+  }
+
+  public void testPropertiesStaticMethods() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "@Property "
+            + "public class FooBar {"
+            + "  "
+            + "  public static String getFieldFoo() {"
+            + "     return \"\";"
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertTranslatedLines(
+        translation,
+        "@property (class, nonatomic, getter=getFieldFoo, readonly) NSString * fieldFoo;");
+  }
+
+  public void testPropertiesOfGetTypesDuplicateNames() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "@Property "
+            + "public class FooBar {"
+            + "  @Property"
+            + "  public String fieldFoo = \"test\";"
+            + "  "
+            + "  "
+            + "  public String getFieldFoo() {"
+            + "     return fieldFoo;"
+            + "  }"
+            + "  "
+            + "  public void setFieldFoo(String fooField) {"
+            + "     this.fieldFoo = fooField;"
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertTranslatedLines(
+        translation,
+        "@property (copy, nonatomic, getter=getFieldFoo, setter=setFieldFooWithNSString:) NSString"
+            + " *fieldFoo;");
+  }
+
+  public void testPropertiesOfGetTypesWithSettersNullable() throws IOException {
+    options.setNullMarked(true);
+    options.setNullability(true);
+    addSourceFile(
+        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+
+    String sourceContent =
+        "package foo.bar; "
+            + "import com.google.j2objc.annotations.Property;"
+            + "import javax.annotation.*;"
+            + "public class FooBar {"
+            + "  private String fieldFoo = \"test\";"
+            + "  "
+            + "  @Property"
+            + "  @Nullable"
+            + "  public String getFooField() {"
+            + "     return fieldFoo;"
+            + "  }"
+            + "  "
+            + "  public void setFooField(@Nullable String fooField) {"
+            + "    this.fieldFoo = fooField;"
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "foo.bar.FooBar", "foo/bar/FooBar.h");
+    assertTranslatedLines(
+        translation,
+        "@property (nonatomic, getter=getFooField, setter=setFooFieldWithNSString:, nullable)"
+            + " NSString * fooField;");
+  }
+
+  public void testPropertiesOfClassTypes() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "@Property "
+            + "public class FooBar {"
+            + "  private String fieldFoo = \"test\";"
+            + "  "
+            + "  public String getFooField() {"
+            + "     return fieldFoo;"
+            + "  }"
+            + "  "
+            + "  public void setFooField(String fooField) {"
+            + "     this.fieldFoo = fooField;"
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertTranslatedLines(
+        translation,
+        "@property (nonatomic, getter=getFooField, setter=setFooFieldWithNSString:)"
+            + " NSString * fooField;");
+  }
+
   public void testAddIgnoreDeprecationWarningsPragmaIfDeprecatedDeclarationsIsEnabled()
       throws IOException {
     options.enableDeprecatedDeclarations();
@@ -734,6 +936,53 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertOccurrences(translation, "J2OBJC_FIELD_SETTER", 1);
   }
 
+  public void testFieldSetterGenerationWithNullMarked() throws IOException {
+    options.setNullMarked(true);
+    options.setNullability(true);
+    addSourceFile(
+        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String translation =
+        translateSourceFile(
+            "package foo.bar;import javax.annotation.*;"
+                + "class Test { @Nullable String a; String b; @Nullable Object c;}",
+            "foo.bar.Test",
+            "foo/bar/Test.h");
+    assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, a_, NSString *_Nullable)");
+    assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, b_, NSString *)");
+    assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, c_, id)");
+  }
+
+  public void testStaticFieldObject() throws IOException {
+    String translation =
+        translateSourceFile(
+            "class Test { public static final String str = \"str\"; }", "Test", "Test.h");
+    assertTranslation(translation, "J2OBJC_STATIC_FIELD_OBJ_FINAL(Test, str, NSString *)");
+  }
+
+  public void testStaticFieldObjectWithNullMarked() throws IOException {
+    options.setNullMarked(true);
+    options.setNullability(true);
+    addSourceFile(
+        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String translation =
+        translateSourceFile(
+            "package foo.bar; import javax.annotation.*;"
+                + "class Test { public static final String a = \"a\";"
+                + "@Nullable public static final String b = null; }",
+            "foo.bar.Test",
+            "foo/bar/Test.h");
+    assertTranslation(translation, "inline NSString *FooBarTest_get_a(void);");
+    assertTranslation(translation, "FOUNDATION_EXPORT NSString *FooBarTest_a;");
+    assertTranslation(translation, "J2OBJC_STATIC_FIELD_OBJ_FINAL(FooBarTest, a, NSString *)");
+
+    assertTranslation(translation, "inline NSString *_Nullable FooBarTest_get_b(void);");
+    assertTranslation(translation, "FOUNDATION_EXPORT NSString *_Nullable FooBarTest_b;");
+    assertTranslation(
+        translation, "J2OBJC_STATIC_FIELD_OBJ_FINAL(FooBarTest, b, NSString *_Nullable)");
+  }
+
   public void testEnumWithNameAndOrdinalParameters() throws IOException {
     String translation = translateSourceFile(
       "public enum Test { FOO(\"foo\", 3), BAR(\"bar\", 5); "
@@ -893,4 +1142,26 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertTranslation(kytheMetadata, "kythe0");
     assertTranslation(kytheMetadata, "{\"type\":\"anchor_anchor\"");
   }
+
+  // Verifies that properly encoded Kythe metadata and associated pragmas are generated when
+  // using the Kythe mapping flag with a source jar.
+  public void testKytheMetadataMappingsWithSourceJar() throws Exception {
+    options.setEmitKytheMappings(true);
+    addJarFile(
+        "some/path/test.jar", "foo/Test.java", "package foo; class Test { public void test() {}}");
+    runPipeline("some/path/test.jar");
+
+    String translation = getTranslatedFile("foo/Test.h");
+    assertTranslation(translation, "#ifdef KYTHE_IS_RUNNING");
+    assertTranslation(
+        translation, "#pragma kythe_inline_metadata" + " \"This file contains Kythe metadata.\"");
+    assertTranslation(translation, "/* This file contains Kythe metadata.");
+
+    // Verify metadata paths don't contain jar URL paths ...
+    String kytheMetadata = extractKytheMetadata(translation);
+    assertNotInTranslation(kytheMetadata, "jar:file:some/path/test.jar");
+
+    // ... just the source file entry in that jar file.
+    assertTranslation(kytheMetadata, "\"path\":\"foo/Test.java\"");
+  }
 }
