@@ -316,6 +316,17 @@ public class StatementGeneratorTest extends GenerationTest {
     assertTranslation(translation, "return JreLoadStatic(Test, FOO);");
   }
 
+  public void testStaticInnerSubclassAccessingOuterStaticVarStrictFieldLoad() throws IOException {
+    options.setStrictFieldLoad(true);
+    String translation =
+        translateSourceFile(
+            "public class Test { public static final Object FOO = new Object(); "
+                + "static class Inner { Object test() { return FOO; }}}",
+            "Test",
+            "Test.m");
+    assertTranslation(translation, "return JreStrictFieldStrongLoad(JreLoadStaticRef(Test, FOO));");
+  }
+
   public void testReservedIdentifierReference() throws IOException {
     String translation = translateSourceFile(
         "public class Test { public int test(int id) { return id; }}",
@@ -333,22 +344,44 @@ public class StatementGeneratorTest extends GenerationTest {
   }
 
   public void testFieldAccess() throws IOException {
-    String translation = translateSourceFile(
-        "import com.google.j2objc.annotations.Weak;"
-        + "public class Test { "
-        + "  Object i;"
-        + "  @Weak Object j;"
-        + "  Test(Object otherI, Object otherJ) {"
-        + "    i = otherI;"
-        + "    j = otherJ;"
-        + "  }"
-        + "}",
-        "Test", "Test.m");
+    String translation =
+        translateSourceFile(
+            "import com.google.j2objc.annotations.Weak;"
+                + "public class Test { "
+                + "  Object i;"
+                + "  @Weak Object j;"
+                + "  Test(Object otherI, Object otherJ) {"
+                + "    i = otherI;"
+                + "    j = otherJ;"
+                + "  }"
+                + "}",
+            "Test",
+            "Test.m");
     assertTranslation(translation, "JreStrongAssign(&self->i_, otherI);");
     assertTranslation(translation, "j_ = otherJ;");
     assertTranslation(translation, "RELEASE_(i_);");
   }
 
+  public void testFieldAccessStrictField() throws IOException {
+    options.setStrictFieldAssign(true);
+    String translation =
+        translateSourceFile(
+            "import com.google.j2objc.annotations.Weak;"
+                + "public class Test { "
+                + "  Object i;"
+                + "  @Weak Object j;"
+                + "  Test(Object otherI, Object otherJ) {"
+                + "    i = otherI;"
+                + "    j = otherJ;"
+                + "  }"
+                + "}",
+            "Test",
+            "Test.m");
+    assertTranslation(translation, "JreStrictFieldStrongAssign(&self->i_, otherI);");
+    assertTranslation(translation, "j_ = otherJ;");
+    assertTranslation(translation, "JreStrictFieldStrongRelease(&i_);");
+  }
+
   public void testStaticFinalFieldAccessWithParenthesizedExpression() throws IOException {
     String translation = translateSourceFile(
         "public class Test { "
@@ -372,6 +405,21 @@ public class StatementGeneratorTest extends GenerationTest {
     assertTranslation(translation, "return [((JavaLangInteger *) nil_chk(i_)) intValue];");
   }
 
+  public void testInnerInnerClassFieldAccessStrictField() throws IOException {
+    options.setStrictFieldLoad(true);
+    String translation =
+        translateSourceFile(
+            "public class Test { static class One {} static class Two extends Test { "
+                + "Integer i; Two(Integer i) { this.i = i; } int getI() { return i.intValue(); }}}",
+            "Test",
+            "Test.m");
+    assertTranslation(
+        translation, "- (instancetype)initWithJavaLangInteger:(JavaLangInteger *)i {");
+    assertTranslation(
+        translation,
+        "return [((JavaLangInteger *) nil_chk(JreStrictFieldStrongLoad(&i_))) intValue];");
+  }
+
   public void testInnerClassSuperConstructor() throws IOException {
     String translation = translateSourceFile(
         "public class Test { static class One { int i; One(int i) { this.i = i; }} "
@@ -557,14 +605,15 @@ public class StatementGeneratorTest extends GenerationTest {
     List<Statement> stmts = translateStatements(source);
     assertEquals(2, stmts.size());
     String result = generateStatement(stmts.get(1));
-    assertTranslatedLines(result,
+    assertTranslatedLines(
+        result,
         "{",
-          "IOSObjectArray *a__ = strings;",
-          "NSString * const *b__ = a__->buffer_;",
-          "NSString * const *e__ = b__ + a__->size_;",
-          "while (b__ < e__) {",
-            "NSString *string = *b__++;",
-          "}",
+        "IOSObjectArray *a__ = strings;",
+        "NSString * const *b__ = a__->buffer_;",
+        "NSString * const *e__ = b__ + a__->size_;",
+        "while (b__ < e__) {",
+        "NSString *string = RETAIN_AND_AUTORELEASE(*b__++);",
+        "}",
         "}");
   }
 
@@ -624,13 +673,26 @@ public class StatementGeneratorTest extends GenerationTest {
   }
 
   public void testNewFieldNotRetained() throws IOException {
-    String translation = translateSourceFile(
-        "import java.util.*; public class A { Map map; A() { map = new HashMap(); }}",
-        "A", "A.m");
+    String translation =
+        translateSourceFile(
+            "import java.util.*; public class A { Map map; A() { map = new HashMap(); }}",
+            "A",
+            "A.m");
     assertTranslation(translation,
         "JreStrongAssignAndConsume(&self->map_, new_JavaUtilHashMap_init())");
   }
 
+  public void testCreatedFieldNotConsumedStrictFieldAssign() throws IOException {
+    options.setStrictFieldAssign(true);
+    String translation =
+        translateSourceFile(
+            "import java.util.*; public class A { Map map; A() { map = new HashMap(); }}",
+            "A",
+            "A.m");
+    assertTranslation(
+        translation, "JreStrictFieldStrongAssign(&self->map_, create_JavaUtilHashMap_init())");
+  }
+
   public void testStringAddOperator() throws IOException {
     String translation = translateSourceFile(
         "import java.util.*; public class A { String myString;"
