@@ -82,6 +82,52 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     assertNotInTranslation(translation, "+ (void)setDEFAULT:(Test *)value");
   }
 
+  // Verify accessor methods are properly annotated for nullability when the class
+  // is annotated with @NullMarked.
+  public void testStaticFieldAccessorMethodWithNullMarkedClass() throws IOException {
+    String source =
+        "import javax.annotation.*; import org.jspecify.nullness.NullMarked;"
+            + "@NullMarked public class Test {"
+            + "  @Nullable public static String nullableStr;"
+            + "  public static String nonNullStr;"
+            + "  Test() {}"
+            + "}";
+    options.setClassProperties(false);
+    options.setStaticAccessorMethods(true);
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "Test", "Test.h");
+    assertTranslation(translation, "+ (NSString *_Nullable)nullableStr;");
+    assertTranslation(translation, "+ (void)setNullableStr:(NSString *_Nullable)value;");
+    assertTranslation(translation, "+ (NSString *)nonNullStr;");
+    assertTranslation(translation, "+ (void)setNonNullStr:(NSString *)value;");
+  }
+
+  // Verify accessor methods are properly annotated for nullability when the package
+  // is annotated with @NullMarked.
+  public void testStaticFieldAccessorMethodWithNullMarkedPackage() throws IOException {
+    addSourceFile(
+        "@NullMarked package foo.bar; import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar; "
+            + "import javax.annotation.*;"
+            + "public class Test {"
+            + "  @Nullable public static String nullableStr;"
+            + "  public static String nonNullStr;"
+            + "  Test() {}"
+            + "}";
+    options.setClassProperties(false);
+    options.setStaticAccessorMethods(true);
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
+    assertTranslation(translation, "+ (NSString *_Nullable)nullableStr;");
+    assertTranslation(translation, "+ (void)setNullableStr:(NSString *_Nullable)value;");
+    assertTranslation(translation, "+ (NSString *)nonNullStr;");
+    assertTranslation(translation, "+ (void)setNonNullStr:(NSString *)value;");
+  }
+
   // Verify that accessor methods for static vars and constants aren't generated by default.
   public void testNoStaticFieldAccessorMethods() throws IOException {
     String source = "class Test { "
@@ -124,10 +170,45 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     assertTranslation(translation, "+ (Test *)EOF_;");
   }
 
+  // Verify that accessor methods for enum constants are generated with nonnull
+  // annotations.
+  public void testEnumConstantAccessorMethodsWithNullability() throws IOException {
+    options.setStaticAccessorMethods(true);
+    options.setNullability(true);
+    String source = "enum Test { ONE, TWO }";
+    String translation = translateSourceFile(source, "Test", "Test.h");
+    assertNotInTranslation(translation, "NS_ASSUME_NONNULL_BEGIN");
+    assertTranslation(translation, "+ (Test * _Nonnull)ONE;");
+    assertTranslation(translation, "+ (Test * _Nonnull)TWO;");
+    assertNotInTranslation(translation, "NS_ASSUME_NONNULL_END");
+  }
+
+  // Verify that accessor methods for enum constants are generated on request with
+  // proper nullability annotations.
+  public void testEnumConstantAccessorMethodsAreImplicitlyNonnullWhenNullMarked()
+      throws IOException {
+    options.setStaticAccessorMethods(true);
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String source =
+        "import javax.annotation.*;"
+            + "@Nullable enum NullableTest { ONE, TWO }"
+            + "enum NonnullTest { THREE, FOUR }";
+    String translation = translateSourceFile(source, "Test", "Test.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface NullableTest : JavaLangEnum");
+    assertTranslation(translation, "+ (NullableTest * _Nullable)ONE;");
+    assertTranslation(translation, "+ (NullableTest * _Nullable)TWO;");
+    assertTranslation(translation, "+ (NonnullTest *)THREE;");
+    assertTranslation(translation, "+ (NonnullTest *)FOUR;");
+    assertTranslation(translation, "NS_ASSUME_NONNULL_END");
+  }
+
   // Verify that class properties for enum constants are generated on request.
   public void testEnumConstantClassProperties() throws IOException {
     options.setClassProperties(true);
     options.setNullability(true);
+    options.setSwiftEnums(false);
     String source = "enum Test { ONE, TWO, EOF }"; // EOF is a reserved name.
     String translation = translateSourceFile(source, "Test", "Test.h");
     assertTranslation(
@@ -141,19 +222,34 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
   // Verify that accessor methods for enum constants are generated by --swift-friendly flag.
   public void testSwiftFriendlyEnumConstantAccessorMethods() throws IOException {
     options.setSwiftFriendly(true);
-    String source = "enum Test { ONE, TWO, EOF }";  // EOF is a reserved name.
+    String source = "enum Test { ONE, TWO, EOF, TWO_WORDS }"; // EOF is a reserved name.
     String translation = translateSourceFile(source, "Test", "Test.h");
-    assertTranslation(translation,
-        "@property (readonly, class, nonnull) Test *ONE NS_SWIFT_NAME(ONE);");
-    assertTranslation(translation,
-        "@property (readonly, class, nonnull) Test *TWO NS_SWIFT_NAME(TWO);");
-    assertTranslation(translation,
-        "@property (readonly, class, nonnull) Test *EOF_ NS_SWIFT_NAME(EOF_);");
+    assertTranslation(
+        translation, "@property (readonly, class, nonnull) Test *ONE NS_SWIFT_NAME(one);");
+    assertTranslation(
+        translation, "@property (readonly, class, nonnull) Test *TWO NS_SWIFT_NAME(two);");
+    assertTranslation(
+        translation, "@property (readonly, class, nonnull) Test *EOF_ NS_SWIFT_NAME(eof);");
+    assertTranslation(
+        translation,
+        "@property (readonly, class, nonnull) Test *TWO_WORDS NS_SWIFT_NAME(twoWords);");
 
     translation = getTranslatedFile("Test.m");
     assertTranslation(translation, "+ (Test *)ONE ");
     assertTranslation(translation, "+ (Test *)TWO ");
     assertTranslation(translation, "+ (Test *)EOF_ ");
+    assertTranslation(translation, "+ (Test *)TWO_WORDS ");
+  }
+
+  // Verify that NS_SWIFT_NAME is added to emums when --swift-enum flag is enabled.
+  public void testSwiftEnumSwiftName() throws IOException {
+    options.setSwiftEnums(true);
+    String source = "enum Test { ONE, TWO, EOF, TWO_WORDS }"; // EOF is a reserved name.
+    String translation = translateSourceFile(source, "Test", "Test.h");
+    assertTranslation(translation, "Test_Enum_ONE NS_SWIFT_NAME(one) = 0,");
+    assertTranslation(translation, "Test_Enum_TWO NS_SWIFT_NAME(two) = 1,");
+    assertTranslation(translation, "Test_Enum_EOF NS_SWIFT_NAME(eof) = 2,");
+    assertTranslation(translation, "Test_Enum_TWO_WORDS NS_SWIFT_NAME(twoWords) = 3,");
   }
 
   // Verify that accessor methods for enum constants are not generated by default.
@@ -254,7 +350,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  @Property(\"cause_exception\") private int fieldBar;"
         + "}";
     translateSourceFile(source, "FooBar", "FooBar.h");
-    assertErrorCount(1);
+    assertError("FooBar.java:1: Invalid @Property attribute: cause_exception");
   }
 
   public void testPropertySetterSelector() throws IOException {
@@ -273,7 +369,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  @Property(\"setter=needs_colon\") private int fieldBar;"
         + "}";
     translateSourceFile(source, "FooBar", "FooBar.h");
-    assertErrorCount(1);
+    assertError("FooBar.java:1: Non-existent setter specified: needs_colon");
   }
 
   public void testNonexistentPropertySetter() throws IOException {
@@ -282,7 +378,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  @Property(\"setter=nonexistent:\") private int fieldBar;"
         + "}";
     translateSourceFile(source, "FooBar", "FooBar.h");
-    assertErrorCount(1);
+    assertError("FooBar.java:1: Non-existent setter specified: nonexistent:");
   }
 
   public void testPropertyWeakAssignment() throws IOException {
@@ -315,7 +411,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  @Property(\"strong\") @Weak Foo barA;"
         + "}";
     translateSourceFile(source, "Foo", "Foo.h");
-    assertErrorCount(1);
+    assertError("Foo.java:1: Weak field annotation conflicts with strong Property attribute");
   }
 
   public void testStrongProperties() throws IOException {
@@ -356,7 +452,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         "import com.google.j2objc.annotations.Property; "
         + "public class Test {  "
         + "@Property(\"class\") int test; }", "Test", "Test.h");
-    assertErrorCount(1);
+    assertError("Test.java:-1: Only static fields can be translated to class properties");
 
     // Verify static accessor generation must be enabled for class properties.
     ErrorUtil.reset();
@@ -365,7 +461,9 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         "import com.google.j2objc.annotations.Property; "
         + "public class Test {  "
         + "@Property static int test; }", "Test", "Test.h");
-    assertErrorCount(1);
+    assertError(
+        "Test.java:-1: Class properties require any of these flags: --swift-friendly,"
+            + " --class-properties or --static-accessor-methods");
 
     // Verify class properties are not supported for private static fields.
     ErrorUtil.reset();
@@ -374,7 +472,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         "import com.google.j2objc.annotations.Property; "
             + "public class Test {  "
             + "@Property private static int test; }", "Test", "Test.h");
-    assertErrorCount(1);
+    assertError("Test.java:-1: Properties are not supported for private static fields.");
   }
 
   public void testClassPropertiesFlag() throws IOException {
@@ -436,16 +534,17 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     String translation = translateSourceFile(source, "Test", "Test.h");
 
     // Verify return type and parameters are all annotated.
-    assertTranslatedLines(translation,
-        "- (NSString * __nullable)testWithNSString:(NSString * __nonnull)msg",
+    assertTranslatedLines(
+        translation,
+        "- (NSString * _Nullable)testWithNSString:(NSString * _Nonnull)msg",
         // var is also nonnull because of the default annotation on the class.
-        "withId:(id __nonnull)var;");
+        "withId:(id _Nonnull)var;");
 
     // Verify return type isn't annotated, as only parameters should be by default.
     assertTranslatedLines(translation, "- (NSString *)test2;");
 
     // Verify return type is annotated.
-    assertTranslation(translation, "- (NSString * __nonnull)test3;");
+    assertTranslation(translation, "- (NSString * _Nonnull)test3;");
   }
 
   // Verify ParametersAreNonnullByDefault sets unspecified parameter as non-null.
@@ -458,11 +557,12 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     options.setNullability(true);
     String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
     // var is also nonnull because of the default annotation on the class.
-    assertTranslatedLines(translation,
+    assertTranslatedLines(
+        translation,
         // Verify parameter isn't affected by default.
-        "- (NSString * __nullable)testWithNSString:(NSString * __nullable)msg",
+        "- (NSString * _Nullable)testWithNSString:(NSString * _Nullable)msg",
         // Verify default nonnull is specified.
-        "withId:(id __nonnull)var",
+        "withId:(id _Nonnull)var",
         // Default should not apply to primitive type.
         "withInt:(jint)count;");
   }
@@ -480,11 +580,12 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     options.setNullability(true);
     String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
     // var is also nonnull because of the default annotation on the class.
-    assertTranslatedLines(translation,
+    assertTranslatedLines(
+        translation,
         // Verify parameter isn't affected by default.
-        "- (NSString * __nullable)testWithNSString:(NSString * __nullable)msg",
+        "- (NSString * _Nullable)testWithNSString:(NSString * _Nullable)msg",
         // Verify default nonnull is specified.
-        "withId:(id __nonnull)var",
+        "withId:(id _Nonnull)var",
         // Default should not apply to primitive type.
         "withInt:(jint)count;");
   }
@@ -552,23 +653,27 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
   }
 
   public void testPropertyNullability() throws IOException {
-    String source = "import javax.annotation.*;"
-        + "import com.google.j2objc.annotations.Property;"
-        + "@ParametersAreNonnullByDefault public class Test {"
-        + "  @Nullable @Property String test;"
-        + "  @Property String test2;"
-        + "  @Property @Nonnull String test3;"
-        + "  @Property(\"nonatomic\") String test4;"
-        + "  @Property(\"null_resettable\") String test5;"
-        + "  @Property(\"null_unspecified\") String test6;"
-        + "  @Property int test7;"
-        + "  @Property (\"readonly, nonatomic\") double test8;"
-        + "}";
+    String source =
+        "import javax.annotation.*;"
+            + "import com.google.j2objc.annotations.Property;"
+            + "@ParametersAreNonnullByDefault public class Test {"
+            + "  @Nullable @Property String test;"
+            + "  @CheckForNull @Property String test1;"
+            + "  @Property String test2;"
+            + "  @Property @Nonnull String test3;"
+            + "  @Property(\"nonatomic\") String test4;"
+            + "  @Property(\"null_resettable\") String test5;"
+            + "  @Property(\"null_unspecified\") String test6;"
+            + "  @Property int test7;"
+            + "  @Property (\"readonly, nonatomic\") double test8;"
+            + "}";
     options.setNullability(true);
     String translation = translateSourceFile(source, "Test", "Test.h");
 
-    assertTranslatedLines(translation,
+    assertTranslatedLines(
+        translation,
         "@property (copy, nullable) NSString *test;",
+        "@property (copy, nullable) NSString *test1;",
         "@property (copy) NSString *test2;",
         "@property (copy, nonnull) NSString *test3;",
         "@property (copy, nonatomic) NSString *test4;");
@@ -592,7 +697,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  int i; "
         + "  public Test() { this(0); } "
         + "  private Test(int i) { this.i = i; }}", "Test", "Test.h");
-    assertTranslation(translation, "- (instancetype __nonnull)init;");
+    assertTranslation(translation, "- (instancetype _Nonnull)init;");
     translation = getTranslatedFile("Test.m");
     assertTranslation(translation, "- (instancetype)initWithInt:(jint)i;");
   }
@@ -614,23 +719,138 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     options.setNullability(true);
     String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
     assertTranslatedLines(
-        translation, "- (NSString * __nullable)testWithNSString:(NSString * __nonnull)msg");
+        translation, "- (NSString * _Nullable)testWithNSString:(NSString * _Nonnull)msg");
   }
 
-  public void testDefaultNonnull() throws IOException {
-    String source = "package foo.bar; import javax.annotation.*; "
-        + "public class Test {"
-        + "  String test(@Nullable String msg, Object var, int count) { "
-        + "    return msg.isEmpty() ? null : msg; }"
-        + "}";
-    options.setDefaultNonnull(true);
+  /*
+   * Temporarily, Guava is annotating usages of type variable's like `E next()` with a
+   * `@ParametricNullness` annotation. This annotation means "the value may be null for an
+   * `Iterator<@Nullable Foo>` but not for an `Iterator<@Nonnull Foo>`." For our purposes, that
+   * means "It might be null."
+   *
+   * TODO(b/188212777): Understand Guava's new nullness annotations on a deeper level so that we can
+   * determine whether a usage of a type variable might or might not be null based on whether it is
+   * declared as `class Iterator<E extends @Nullable Object> { ... }` or `class Iterator<E> { ...
+   * }`. Then, Guava can remove @ParametricNullness (and also replace its usages of
+   * @ParametersAreNonnullByDefault with usages of a new annotation).
+   */
+  public void testGuavaTransitionalParametricNullnessAnnotation() throws IOException {
+    String source =
+        "package foo.bar; import javax.annotation.*; "
+            + "@java.lang.annotation.Target(value={java.lang.annotation.ElementType.TYPE_USE})"
+            + "@interface Nullable {}"
+            + "@interface ParametricNullness {}"
+            + "@ParametersAreNonnullByDefault public class Test<T extends @Nullable Object> {"
+            + "  @ParametricNullness T passthrough(@ParametricNullness T t) { "
+            + "    return t; }"
+            + "}";
+    options.setNullability(true);
     String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
-    assertTranslatedLines(translation,
-        "NS_ASSUME_NONNULL_BEGIN",
-        "@interface FooBarTest : NSObject");
-    assertTranslatedLines(translation,
-        "@end",
-        "NS_ASSUME_NONNULL_END");
+    assertTranslatedLines(translation, "- (id _Nullable)passthroughWithId:(id _Nullable)t;");
+  }
+
+  public void testNullMarked() throws IOException {
+    addSourceFile(
+        "@NullMarked package foo.bar; import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar; import javax.annotation.*; "
+            + "public class Test {  "
+            + "  public @Nullable String identifier; "
+            + "  private final @Nullable String token; "
+            + "  Test(@Nullable String token) { this.token = token; }"
+            + "  public @Nullable String getToken() { return token; } "
+            + "  @Nullable String testNullableInstanceMethod(@Nullable String msg,"
+            + "    Object var, int count) {"
+            + "      return msg.isEmpty() ? null : msg;"
+            + "  } "
+            + "  public static @Nullable String testNullableStaticMethod(@Nullable String firstArg,"
+            + "    Object var, int count) {"
+            + "      return null;"
+            + "    }"
+            + "}";
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest : NSObject {");
+    assertTranslatedLines(translation, "@public", "NSString *_Nullable identifier_;");
+    assertTranslation(translation, "- (NSString * _Nullable)getToken;");
+    assertTranslation(
+        translation, "- (instancetype _Nonnull)initWithNSString:(NSString * _Nullable)token;");
+    assertTranslatedLines(
+        translation,
+        "- (NSString * _Nullable)testNullableInstanceMethodWithNSString:(NSString *"
+            + " _Nullable)msg",
+        "withId:(id)var",
+        "withInt:(jint)count;");
+    assertTranslatedLines(
+        translation,
+        "+ (NSString * _Nullable)testNullableStaticMethodWithNSString:(NSString *"
+            + " _Nullable)firstArg",
+        "withId:(id)var",
+        "withInt:(jint)count;");
+    assertTranslatedLines(
+        translation, "J2OBJC_TYPE_LITERAL_HEADER(FooBarTest)", "", "NS_ASSUME_NONNULL_END");
+  }
+
+  public void testNullMarkedTypeAnnotation() throws IOException {
+    String source =
+        "import javax.annotation.*;"
+            + "import org.jspecify.nullness.NullMarked;"
+            + "@NullMarked public class Test {"
+            + "  private @Nullable String nullableStr;"
+            + "  private String nonNullStr;"
+            + "  public @Nullable String test(String a,"
+            + "    @Nullable String b) {"
+            + "      return null;"
+            + "  }"
+            + "  public @Nullable String getNullableStr() { return nullableStr; }"
+            + "  public String getNonNullStr() { return nonNullStr; }"
+            + "}";
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "Test", "Test.h");
+    assertTranslatedLines(translation, "NS_ASSUME_NONNULL_BEGIN", "@interface Test : NSObject");
+    assertTranslatedLines(
+        translation,
+        "- (NSString * _Nullable)testWithNSString:(NSString *)a",
+        "withNSString:(NSString * _Nullable)b;");
+    assertTranslation(translation, "- (NSString *)getNonNullStr;");
+    assertTranslation(translation, "- (NSString * _Nullable)getNullableStr;");
+    assertTranslation(translation, "NS_ASSUME_NONNULL_END");
+  }
+
+  public void testNullMarkedPackageAnnotation() throws IOException {
+    addSourceFile(
+        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar;"
+            + "import javax.annotation.*;"
+            + "import org.jspecify.nullness.NullMarked;"
+            + "@NullMarked public class Test {"
+            + "  private @Nullable String nullableStr;"
+            + "  private String nonNullStr;"
+            + "  public @Nullable String test(String a,"
+            + "    @Nullable String b) {"
+            + "      return null;"
+            + "  }"
+            + "  public @Nullable String getNullableStr() { return nullableStr; }"
+            + "  public String getNonNullStr() { return nonNullStr; }"
+            + "}";
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest : NSObject");
+    assertTranslatedLines(
+        translation,
+        "- (NSString * _Nullable)testWithNSString:(NSString *)a",
+        "withNSString:(NSString * _Nullable)b;");
+    assertTranslation(translation, "- (NSString *)getNonNullStr;");
+    assertTranslation(translation, "- (NSString * _Nullable)getNullableStr;");
+    assertTranslation(translation, "NS_ASSUME_NONNULL_END");
   }
 
   public void testFieldWithIntersectionType() throws IOException {
@@ -782,4 +1002,18 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
       fail("No mapping found for MyClass() constructor");
     }
   }
+
+  // https://github.com/google/j2objc/issues/1664
+  public void testStaticInitializeField() throws IOException {
+    String translation = translateSourceFile("final class State\n"
+        + "{\n"
+        + "    public static final int initialize=0;\n"
+        + "    public static final int ready=1;\n"
+        + "    public static final int anotherState=2;\n"
+        + "}\n", "State", "State.h");
+    // Should have trailing underscore, to avoid class with class initialize function name.
+    assertTranslation(translation, "J2OBJC_STATIC_FIELD_CONSTANT(State, initialize_, jint)");
+    assertTranslation(translation, "inline jint State_get_initialize_(void);");
+    assertTranslation(translation, "State_initialize_");
+  }
 }
