@@ -18,7 +18,6 @@ package com.google.devtools.j2objc.translate;
 
 import com.google.devtools.j2objc.GenerationTest;
 import com.google.devtools.j2objc.util.CodeReferenceMap;
-
 import java.io.IOException;
 
 /**
@@ -202,6 +201,20 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertTranslation(translation, "Foo_Bar_init");
   }
 
+ public void testDeadClass_NestedEnum() throws IOException {
+    CodeReferenceMap map = CodeReferenceMap.builder()
+        .addClass("Foo")
+        .build();
+    setDeadCodeMap(map);
+    String source = "class Foo {\n"
+        + "  enum Bar {BAZ}\n"
+        + "}\n";
+    String translation = translateSourceFile(source, "Foo", "Foo.h");
+    assertTranslation(translation, "@interface Foo_Bar : JavaLangEnum");
+    translation = getTranslatedFile("Foo.m");
+    assertTranslation(translation, "Foo_Bar *Foo_Bar_values_[1]");
+  }
+
   public void testDeadClass_DeadStaticNestedClass() throws IOException {
     CodeReferenceMap map = CodeReferenceMap.builder()
         .addClass("Foo")
@@ -225,7 +238,7 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertNotInTranslation(translation, "- (void)g");
   }
 
-  public void testDeadClass_DeadInnerClassConstructor() throws IOException {
+  public void testDeadClass_deadInnerClassConstructor() throws IOException {
     CodeReferenceMap map = CodeReferenceMap.builder()
         .addField("Foo$A", "z")
         .addField("Foo$A", "this$0")
@@ -253,7 +266,7 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertNotInTranslation(translation, "- (jint)f");
   }
 
-  public void testDeadClass_SupertypeRemoval() throws IOException {
+  public void testDeadClass_supertypeRemoval() throws IOException {
     CodeReferenceMap map = CodeReferenceMap.builder()
         .addClass("Foo")
         .build();
@@ -268,7 +281,7 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertNotInTranslation(translation, "SuperI");
   }
 
-  // Verify that annotation bodies aren't stripped when specified in a dead code report.
+  // Verifies that annotation bodies aren't stripped when specified in a dead code report.
   public void testDeadAnnotation() throws IOException {
     CodeReferenceMap map = CodeReferenceMap.builder()
         .addClass("Foo")
@@ -284,6 +297,52 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertTranslation(translation, "@property (readonly) NSString *value;");
   }
 
+  // Verifies that RUNTIME annotations are stripped when the strip-reflection flag is on
+  public void testDeadAnnotation_removeRuntimeAnnotation_whenStripReflectionIsOn()
+      throws IOException {
+    options.setStripReflection(true);
+    CodeReferenceMap map = CodeReferenceMap.builder().addClass("Foo").build();
+    setDeadCodeMap(map);
+    String source =
+        "import java.lang.annotation.Retention;\n"
+            + "import java.lang.annotation.RetentionPolicy;\n"
+            + "@Retention(RetentionPolicy.RUNTIME)\n"
+            + "public @interface Foo {\n"
+            + "  String value() default \"\";\n"
+            + "}\n";
+    String translation = translateSourceFile(source, "Foo", "Foo.h");
+    assertNotInTranslation(translation, "@protocol Foo < JavaLangAnnotationAnnotation >");
+  }
+
+  // Verifies that default (CLASS) annotations are stripped.
+  public void testDeadAnnotation_removeDefaultClassAnnotation() throws IOException {
+    CodeReferenceMap map = CodeReferenceMap.builder().addClass("Foo").build();
+    setDeadCodeMap(map);
+    String source =
+        "import java.lang.annotation.Retention;\n"
+            + "import java.lang.annotation.RetentionPolicy;\n"
+            + "public @interface Foo {\n"
+            + "  String value() default \"\";\n"
+            + "}\n";
+    String translation = translateSourceFile(source, "Foo", "Foo.h");
+    assertNotInTranslation(translation, "@protocol Foo < JavaLangAnnotationAnnotation >");
+  }
+
+  // Verifies that CLASS annotations are stripped.
+  public void testDeadAnnotation_removeClassAnnotation() throws IOException {
+    CodeReferenceMap map = CodeReferenceMap.builder().addClass("Foo").build();
+    setDeadCodeMap(map);
+    String source =
+        "import java.lang.annotation.Retention;\n"
+            + "import java.lang.annotation.RetentionPolicy;\n"
+            + "@Retention(RetentionPolicy.CLASS)\n"
+            + "public @interface Foo {\n"
+            + "  String value() default \"\";\n"
+            + "}\n";
+    String translation = translateSourceFile(source, "Foo", "Foo.h");
+    assertNotInTranslation(translation, "@protocol Foo < JavaLangAnnotationAnnotation >");
+  }
+
   public void testDeadDefaultConstructor() throws IOException {
     CodeReferenceMap map = CodeReferenceMap.builder()
         .addMethod("Test", "Test", "()V")
@@ -557,4 +616,52 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertTranslation(translation, "NSString *A_B_FOO = @\"foo\";");
     assertTranslation(translation, "static NSString *A_B_BAR = @\"bar\";");
   }
+
+  public void testDeadClass_stringConstants_nullMarked() throws IOException {
+    addSourceFile(
+        "@NullMarked" + "package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar;"
+            + "import javax.annotation.*;"
+            + "class A {"
+            + "  public static class B {"
+            + "    @Nullable public static final String FOO = \"foo\";"
+            + "    private static final String BAR = \"bar\";"
+            + "  }"
+            + "  @Nullable String test() {"
+            + "    return B.FOO + B.BAR;"
+            + "  }"
+            + "}";
+    CodeReferenceMap map = CodeReferenceMap.builder().addClass("A$B").build();
+    setDeadCodeMap(map);
+    // Check that accessors from a dead class are not generated.
+    options.setStaticAccessorMethods(true);
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "foo.bar.A", "foo/bar/A.h");
+    assertTranslation(translation, "FOUNDATION_EXPORT NSString *_Nullable FooBarA_B_FOO;");
+
+    translation = getTranslatedFile("foo/bar/A.m");
+    assertTranslation(translation, "NSString *_Nullable FooBarA_B_FOO = @\"foo\";");
+    assertTranslation(translation, "static NSString *FooBarA_B_BAR = @\"bar\";");
+  }
+
+  // Verify when an outer type is dead, a live inner type is still generated. b/289536498
+  public void testDeadOuterInterface_liveAnnotation() throws IOException {
+    CodeReferenceMap map =
+        CodeReferenceMap.builder().addClass("Foo").addClass("Foo$Inner2").build();
+    setDeadCodeMap(map);
+    String source = "interface Foo {  public @interface Inner {} public @interface Inner2 {} }";
+    String translation = translateSourceFile(source, "Foo", "Foo.h");
+    assertNotInTranslation(translation, "@protocol Foo < JavaObject >");
+    assertTranslation(translation, "@protocol Foo_Inner < JavaLangAnnotationAnnotation >");
+    assertTranslation(translation, "@interface Foo_Inner : NSObject < Foo_Inner >");
+    assertNotInTranslation(translation, "@protocol Foo_Inner2 < JavaObject >");
+
+    translation = getTranslatedFile("Foo.m");
+    assertNotInTranslation(translation, "J2OBJC_INTERFACE_TYPE_LITERAL_SOURCE(Foo)");
+    assertTranslation(translation, "J2OBJC_INTERFACE_TYPE_LITERAL_SOURCE(Foo_Inner)");
+    assertNotInTranslation(translation, "J2OBJC_INTERFACE_TYPE_LITERAL_SOURCE(Foo_Inner2)");
+  }
 }
