@@ -15,6 +15,7 @@
 package com.google.devtools.j2objc.translate;
 
 import com.google.devtools.j2objc.GenerationTest;
+import com.google.devtools.j2objc.util.ErrorUtil;
 import java.io.IOException;
 
 /**
@@ -67,8 +68,8 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
             + "  public String baz() { return null; } " // no external annotation.
             + "}";
     String translation = translateSourceFile(source, "p.Test", "p/Test.h");
-    assertTranslation(translation, "- (NSString * __nonnull)foo;");
-    assertTranslation(translation, "- (NSString * __nullable)bar;");
+    assertTranslation(translation, "- (NSString * _Nonnull)foo;");
+    assertTranslation(translation, "- (NSString * _Nullable)bar;");
     assertTranslation(translation, "- (NSString *)baz;");
     assertNotInTranslation(translation, "qux");
   }
@@ -82,8 +83,8 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
             + "  public static String bar() { return null; } "
             + "}";
     String translation = translateSourceFile(source, "p.Test", "p/Test.h");
-    assertTranslation(translation, "+ (NSString * __nonnull)foo;");
-    assertTranslation(translation, "+ (NSString * __nullable)bar;");
+    assertTranslation(translation, "+ (NSString * _Nonnull)foo;");
+    assertTranslation(translation, "+ (NSString * _Nullable)bar;");
   }
 
   public void testInjectNullability_returnType_enumMethod() throws IOException {
@@ -96,8 +97,8 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
             + "    public String bar() { return null; } "
             + "}";
     String translation = translateSourceFile(source, "p.Test", "p/Test.h");
-    assertTranslation(translation, "- (NSString * __nonnull)foo;");
-    assertTranslation(translation, "- (NSString * __nullable)bar;");
+    assertTranslation(translation, "- (NSString * _Nonnull)foo;");
+    assertTranslation(translation, "- (NSString * _Nullable)bar;");
   }
 
   public void testInjectNullability_returnType_interfaceMethod() throws IOException {
@@ -109,8 +110,8 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
             + "    default String bar() { return null; } "
             + "}";
     String translation = translateSourceFile(source, "p.Test", "p/Test.h");
-    assertTranslation(translation, "- (NSString * __nonnull)foo;");
-    assertTranslation(translation, "- (NSString * __nullable)bar;");
+    assertTranslation(translation, "- (NSString * _Nonnull)foo;");
+    assertTranslation(translation, "- (NSString * _Nullable)bar;");
   }
 
   public void testInjectNullability_returnType_nestedClassMethod() throws IOException {
@@ -132,8 +133,8 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
             + "  }"
             + "}";
     String translation = translateSourceFile(source, "p.Test", "p/Test.h");
-    assertTranslation(translation, "- (NSString * __nonnull)foo;");
-    assertTranslation(translation, "- (NSString * __nullable)bar;");
+    assertTranslation(translation, "- (NSString * _Nonnull)foo;");
+    assertTranslation(translation, "- (NSString * _Nullable)bar;");
   }
 
   // Nullability specifiers should not be applied to primitive types.
@@ -163,7 +164,7 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
     addSourceFile("package p; public @interface AnAnnotation {}", "p/AnAnnotation.java");
     String source = "package p; public class Test { public Test() {} }";
     String translation = translateSourceFile(source, "p.Test", "p/Test.h");
-    assertTranslation(translation, "- (instancetype __nonnull)init;");
+    assertTranslation(translation, "- (instancetype _Nonnull)init;");
     assertTranslation(translation, "FOUNDATION_EXPORT void PTest_init(PTest *self);");
     assertTranslation(
         translation, "FOUNDATION_EXPORT PTest *new_PTest_init(void) NS_RETURNS_RETAINED;");
@@ -275,7 +276,7 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
     // Verify @ObjectiveCName.
     assertTranslation(translation, "@interface XYZTest");
     // Verify @ParametersAreNonnullByDefault.
-    assertTranslation(translation, "- (void)testWithNSString:(NSString * __nonnull)s");
+    assertTranslation(translation, "- (void)testWithNSString:(NSString * _Nonnull)s");
     // Verify @ReflectionSupport.
     translation = getTranslatedFile("p/Test.m");
     assertNotInTranslation(translation, "__metadata");
@@ -326,7 +327,6 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
 
   public void testWeakOuter() throws IOException {
     String source = "package p; "
-        + "import com.google.j2objc.annotations.WeakOuter; "
         + "public class A { "
         + "  Object o; "
         + "  class B { "
@@ -347,7 +347,6 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
 
   public void testWeak() throws IOException {
     String source = "package p; "
-        + "import com.google.j2objc.annotations.Weak;"
         + "public class A { "
         + "  private Thread t;"
         + "  public A(Thread otherT) {"
@@ -367,7 +366,6 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
 
   public void testRetainedWith() throws IOException {
     String source = "package p; "
-        + "import com.google.j2objc.annotations.RetainedWith; "
         + "public class A { "
         + "  private Object o; "
         + "  public void setO(Object o) { this.o = o; } "
@@ -385,7 +383,6 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
 
   public void testAutoreleasePool() throws IOException {
     String source = "package p; "
-        + "import com.google.j2objc.annotations.AutoreleasePool; "
         + "public class A { "
         + "  public void test() {} "
         + "}";
@@ -398,4 +395,21 @@ public class ExternalAnnotationInjectorTest extends GenerationTest {
     String translation = translateSourceFile(source, "p.A", "p/A.m");
     assertTranslation(translation, "@autoreleasepool");
   }
+
+  public void testAnnotationClassNotFound() throws IOException {
+    String source = "package p; "
+        + "public class A { "
+        + "  public void test() {} "
+        + "}";
+    addExternalAnnotationFileContents(
+        "package com.google.j2objc.annotations: "
+            + "annotation @NoSuchAnnotation: "
+            + "package p: "
+            + "class A: "
+            + "  method test()V: @NoSuchAnnotation ");
+    translateSourceFile(source, "p.A", "p/A.m");
+    assertEquals(1, ErrorUtil.errorCount());
+    String errorMsg = ErrorUtil.getErrorMessages().get(0);
+    assertTrue(errorMsg.contains("com.google.j2objc.annotations.NoSuchAnnotation"));
+  }
 }
