@@ -16,7 +16,6 @@ package com.google.devtools.j2objc.translate;
 
 import com.google.devtools.j2objc.GenerationTest;
 import com.google.devtools.j2objc.Options.MemoryManagementOption;
-
 import java.io.IOException;
 
 /**
@@ -512,6 +511,39 @@ public class FunctionizerTest extends GenerationTest {
     innerTestFunctionizedConstructors();
   }
 
+  public void testFunctionizedConstructorWithNullMarked() throws IOException {
+    addSourceFile(
+        "@NullMarked" + "package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar; import javax.annotation.*;"
+            + "public class Test {"
+            + "  @Nullable private final String nullableStr;"
+            + "  private final String nonNullStr;"
+            + "  Test(@Nullable String nullableStr, String nonNullStr) {"
+            + "    this.nullableStr = nullableStr; this.nonNullStr = nonNullStr;"
+            + "   }"
+            + "}";
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest : NSObject");
+    assertTranslation(
+        translation,
+        "FOUNDATION_EXPORT void FooBarTest_initWithNSString_withNSString_(FooBarTest *self,"
+            + " NSString *_Nullable nullableStr, NSString *nonNullStr);");
+    assertTranslation(
+        translation,
+        "FOUNDATION_EXPORT FooBarTest *new_FooBarTest_initWithNSString_withNSString_(NSString"
+            + " *_Nullable nullableStr, NSString *nonNullStr) NS_RETURNS_RETAINED;");
+    assertTranslation(
+        translation,
+        "FOUNDATION_EXPORT FooBarTest *create_FooBarTest_initWithNSString_withNSString_(NSString"
+            + " *_Nullable nullableStr, NSString *nonNullStr);");
+    assertTranslation(translation, "NS_ASSUME_NONNULL_END");
+  }
+
   public void testNoAllocatingConstructorsForAbstractClass() throws IOException {
     String translation = translateSourceFile("abstract class Test {}", "Test", "Test.h");
     assertTranslation(translation, "FOUNDATION_EXPORT void Test_init(Test *self);");
